
#include <glib.h>
#include <glib-object.h>
#include <libgnomenu.h>
#include <stdlib.h>
#include <string.h>
#include <libwnck/libwnck.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <glib/gi18n-lib.h>
#include <float.h>
#include <math.h>


#define TYPE_SWITCHER (switcher_get_type ())
#define SWITCHER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SWITCHER, Switcher))
#define SWITCHER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SWITCHER, SwitcherClass))
#define IS_SWITCHER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SWITCHER))
#define IS_SWITCHER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SWITCHER))
#define SWITCHER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SWITCHER, SwitcherClass))

typedef struct _Switcher Switcher;
typedef struct _SwitcherClass SwitcherClass;
typedef struct _SwitcherPrivate SwitcherPrivate;

#define TYPE_SEARCH_BOX_MENU_ITEM (search_box_menu_item_get_type ())
#define SEARCH_BOX_MENU_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SEARCH_BOX_MENU_ITEM, SearchBoxMenuItem))
#define SEARCH_BOX_MENU_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SEARCH_BOX_MENU_ITEM, SearchBoxMenuItemClass))
#define IS_SEARCH_BOX_MENU_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SEARCH_BOX_MENU_ITEM))
#define IS_SEARCH_BOX_MENU_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SEARCH_BOX_MENU_ITEM))
#define SEARCH_BOX_MENU_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SEARCH_BOX_MENU_ITEM, SearchBoxMenuItemClass))

typedef struct _SearchBoxMenuItem SearchBoxMenuItem;
typedef struct _SearchBoxMenuItemClass SearchBoxMenuItemClass;

#define TYPE_WORKSPACE_SELECTOR (workspace_selector_get_type ())
#define WORKSPACE_SELECTOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_WORKSPACE_SELECTOR, WorkspaceSelector))
#define WORKSPACE_SELECTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_WORKSPACE_SELECTOR, WorkspaceSelectorClass))
#define IS_WORKSPACE_SELECTOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_WORKSPACE_SELECTOR))
#define IS_WORKSPACE_SELECTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_WORKSPACE_SELECTOR))
#define WORKSPACE_SELECTOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_WORKSPACE_SELECTOR, WorkspaceSelectorClass))

typedef struct _WorkspaceSelector WorkspaceSelector;
typedef struct _WorkspaceSelectorClass WorkspaceSelectorClass;
typedef struct _SearchBoxMenuItemPrivate SearchBoxMenuItemPrivate;

struct _Switcher {
	GnomenuMenuBar parent_instance;
	SwitcherPrivate * priv;
};

struct _SwitcherClass {
	GnomenuMenuBarClass parent_class;
};

struct _SwitcherPrivate {
	char* _label;
	gint _max_size;
	gboolean _show_icon;
	gboolean _show_label;
	gboolean _show_window_list;
	gboolean _show_window_actions;
	GnomenuMenuItem* _label_item;
	WnckWindow* _current_window;
	char* NOWIN_TEMPLATE;
	char* NOACT_TEMPLATE;
	char* WINACT_TEMPLATE;
	SearchBoxMenuItem* search_box;
	gboolean disposed;
};

struct _SearchBoxMenuItem {
	GtkImageMenuItem parent_instance;
	SearchBoxMenuItemPrivate * priv;
};

struct _SearchBoxMenuItemClass {
	GtkImageMenuItemClass parent_class;
};

struct _SearchBoxMenuItemPrivate {
	GtkEntry* textbox;
};



GType switcher_get_type (void);
GType search_box_menu_item_get_type (void);
#define SWITCHER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_SWITCHER, SwitcherPrivate))
enum  {
	SWITCHER_DUMMY_PROPERTY,
	SWITCHER_CURRENT_WINDOW,
	SWITCHER_MAX_SIZE,
	SWITCHER_SHOW_ICON,
	SWITCHER_SHOW_LABEL,
	SWITCHER_SHOW_WINDOW_LIST,
	SWITCHER_SHOW_WINDOW_ACTIONS
};
SearchBoxMenuItem* search_box_menu_item_new (void);
SearchBoxMenuItem* search_box_menu_item_construct (GType object_type);
#define SWITCHER_MENU_TEMPLATE "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<menu>\n\t<item id=\"switcher\" underline=\"false\" label=\"%label%\" font=\"bold\" type=\"%type%\" icon=\"%icon%\">\n\t\t\t%submenu%\n\t</item>\n</menu>"
#define SWITCHER_ITEM_TEMPLATE "<item type=\"image\" id=\"XID%id%\" label=\"%label%\" underline=\"false\" font=\"%font%\" icon=\"pixbuf:%pixdata%\" sensitive=\"true\">%submenu%</item>"
Switcher* switcher_new (void);
Switcher* switcher_construct (GType object_type);
Switcher* switcher_new (void);
void switcher_set_current_window (Switcher* self, WnckWindow* value);
static void switcher_real_dispose (GObject* base);
static void switcher_update (Switcher* self, gboolean include_menu);
static void switcher_on_deactivate (Switcher* self);
static WnckWindow* switcher_item_to_window (Switcher* self, GnomenuMenuItem* item);
static gboolean switcher_is_in_sight (Switcher* self, WnckWindow* window);
WorkspaceSelector* workspace_selector_new (WnckWindow* window);
WorkspaceSelector* workspace_selector_construct (GType object_type, WnckWindow* window);
GType workspace_selector_get_type (void);
static void switcher_perhaps_minimize_window (Switcher* self, WnckWindow* window);
static void switcher_on_activate (Switcher* self, GnomenuMenuBar* _this, GnomenuMenuItem* item);
gboolean guess_dock_is_around (void);
static void switcher_set_iconify_destination (Switcher* self, WnckWindow* window);
static char* switcher_do_action_menu (Switcher* self, WnckWindow* window);
char* pixbuf_encode_b64 (GdkPixbuf* pixbuf);
GdkPixbuf* guess_icon (WnckWindow* window, gint width, gint height);
char* template_replace (const char* template, char** subs, int subs_length1);
WnckWindow* find_desktop (void);
static char* switcher_do_xml_menu (Switcher* self);
void search_box_menu_item_set_text (SearchBoxMenuItem* self, const char* value);
WnckWindow* switcher_get_current_window (Switcher* self);
char* replace (const char* source, const char* find, const char* replacement);
static GnomenuMenuItem* switcher_window_to_item (Switcher* self, WnckWindow* window);
gint switcher_get_max_size (Switcher* self);
static void switcher_real_size_request (GtkWidget* base, GtkRequisition* req);
void switcher_set_max_size (Switcher* self, gint value);
gboolean switcher_get_show_icon (Switcher* self);
void switcher_set_show_icon (Switcher* self, gboolean value);
gboolean switcher_get_show_label (Switcher* self);
void switcher_set_show_label (Switcher* self, gboolean value);
gboolean switcher_get_show_window_list (Switcher* self);
void switcher_set_show_window_list (Switcher* self, gboolean value);
gboolean switcher_get_show_window_actions (Switcher* self);
void switcher_set_show_window_actions (Switcher* self, gboolean value);
static void _switcher_on_activate_gnomenu_menu_bar_activate (Switcher* _sender, GnomenuMenuItem* item, gpointer self);
char* template_replace_simple (const char* template, const char* keyword, const char* value);
static void _switcher_on_deactivate_gtk_menu_shell_deactivate (Switcher* _sender, gpointer self);
static GObject * switcher_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer switcher_parent_class = NULL;
static void switcher_finalize (GObject* obj);
static void switcher_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void switcher_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
#define SEARCH_BOX_MENU_ITEM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_SEARCH_BOX_MENU_ITEM, SearchBoxMenuItemPrivate))
enum  {
	SEARCH_BOX_MENU_ITEM_DUMMY_PROPERTY,
	SEARCH_BOX_MENU_ITEM_TEXT
};
SearchBoxMenuItem* search_box_menu_item_new (void);
GnomenuMenuItem* search_box_menu_item_find_item (SearchBoxMenuItem* self);
static gboolean search_box_menu_item_steal_key_press_event (SearchBoxMenuItem* self, GtkWidget* widget, const GdkEventKey* event);
static gboolean _search_box_menu_item_steal_key_press_event_gtk_widget_key_press_event (GtkWidget* _sender, const GdkEventKey* event, gpointer self);
static void search_box_menu_item_real_parent_set (GtkWidget* base, GtkWidget* previous_parent);
static void search_box_menu_item_real_activate_item (GtkMenuItem* base);
static void search_box_menu_item_real_activate (GtkMenuItem* base);
const char* search_box_menu_item_get_text (SearchBoxMenuItem* self);
static void _lambda0_ (GtkEntry* box, GtkStateType previous_state, SearchBoxMenuItem* self);
static void __lambda0__gtk_widget_state_changed (GtkEntry* _sender, GtkStateType previous_state, gpointer self);
static void _lambda1_ (GtkEntry* box, SearchBoxMenuItem* self);
static void __lambda1__gtk_editable_changed (GtkEntry* _sender, gpointer self);
static void _lambda2_ (GtkEntry* box, SearchBoxMenuItem* self);
static void __lambda2__gtk_entry_activate (GtkEntry* _sender, gpointer self);
static void _lambda3_ (SearchBoxMenuItem* _this, GtkStateType previous_state, SearchBoxMenuItem* self);
static void __lambda3__gtk_widget_state_changed (SearchBoxMenuItem* _sender, GtkStateType previous_state, gpointer self);
static GObject * search_box_menu_item_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer search_box_menu_item_parent_class = NULL;
static void search_box_menu_item_finalize (GObject* obj);
static void search_box_menu_item_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void search_box_menu_item_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GdkPixbuf* guess_icon_array (GdkPixbuf** icons, int icons_length1, gint width, gint height);
char* remove_path (const char* txt, const char* separator);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);



Switcher* switcher_construct (GType object_type) {
	Switcher * self;
	self = (Switcher*) gnomenu_menu_bar_construct (object_type);
	return self;
}


Switcher* switcher_new (void) {
	return switcher_construct (TYPE_SWITCHER);
}


static void switcher_real_dispose (GObject* base) {
	Switcher * self;
	self = (Switcher*) base;
	if (!self->priv->disposed) {
		self->priv->disposed = TRUE;
		switcher_set_current_window (self, NULL);
	}
}


static void switcher_on_deactivate (Switcher* self) {
	g_return_if_fail (self != NULL);
	switcher_update (self, FALSE);
}


static void switcher_on_activate (Switcher* self, GnomenuMenuBar* _this, GnomenuMenuItem* item) {
	WnckWindow* window;
	GQuark _tmp8_;
	const char* _tmp7_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (_this != NULL);
	g_return_if_fail (item != NULL);
	if (item == self->priv->_label_item) {
		switcher_update (self, TRUE);
		return;
	}
	/** Then handle the window actions */
	window = switcher_item_to_window (self, item);
	/* if it is an Action Menu item, try to get reference to parent */
	if (window == NULL) {
		char** _tmp3_;
		gint paths_size;
		gint paths_length1;
		char** _tmp1_;
		char* _tmp0_;
		char** _tmp2_;
		char** paths;
		char* parent_path;
		WnckWindow* _tmp6_;
		GnomenuMenuItem* _tmp5_;
		_tmp3_ = NULL;
		_tmp1_ = NULL;
		_tmp0_ = NULL;
		_tmp2_ = NULL;
		paths = (_tmp3_ = (_tmp2_ = _tmp1_ = g_strsplit (_tmp0_ = gnomenu_item_get_item_path ((GnomenuItem*) item), "/", 0), _tmp0_ = (g_free (_tmp0_), NULL), _tmp2_), paths_length1 = _vala_array_length (_tmp1_), paths_size = paths_length1, _tmp3_);
		parent_path = g_strdup ("");
		{
			gint co;
			co = 0;
			for (; co < (paths_length1 - 1); co++) {
				if (_vala_strcmp0 (paths[co], "") != 0) {
					char* _tmp4_;
					_tmp4_ = NULL;
					parent_path = (_tmp4_ = g_strconcat (parent_path, "/", paths[co], NULL, NULL), parent_path = (g_free (parent_path), NULL), _tmp4_);
				}
			}
		}
		_tmp6_ = NULL;
		_tmp5_ = NULL;
		window = (_tmp6_ = switcher_item_to_window (self, _tmp5_ = gnomenu_menu_bar_get ((GnomenuMenuBar*) self, parent_path)), (window == NULL) ? NULL : (window = (g_object_unref (window), NULL)), _tmp6_);
		(_tmp5_ == NULL) ? NULL : (_tmp5_ = (g_object_unref (_tmp5_), NULL));
		paths = (_vala_array_free (paths, paths_length1, (GDestroyNotify) g_free), NULL);
		parent_path = (g_free (parent_path), NULL);
	}
	_tmp7_ = NULL;
	static GQuark _tmp8__label0 = 0;
	static GQuark _tmp8__label1 = 0;
	static GQuark _tmp8__label2 = 0;
	static GQuark _tmp8__label3 = 0;
	static GQuark _tmp8__label4 = 0;
	static GQuark _tmp8__label5 = 0;
	static GQuark _tmp8__label6 = 0;
	static GQuark _tmp8__label7 = 0;
	static GQuark _tmp8__label8 = 0;
	static GQuark _tmp8__label9 = 0;
	static GQuark _tmp8__label10 = 0;
	static GQuark _tmp8__label11 = 0;
	_tmp7_ = gnomenu_item_get_item_id ((GnomenuItem*) item);
	_tmp8_ = (NULL == _tmp7_) ? 0 : g_quark_from_string (_tmp7_);
	if (_tmp8_ == ((0 != _tmp8__label0) ? _tmp8__label0 : (_tmp8__label0 = g_quark_from_static_string ("minimize"))))
	do {
		if (window != NULL) {
			wnck_window_minimize (window);
		}
		break;
	} while (0); else if (_tmp8_ == ((0 != _tmp8__label1) ? _tmp8__label1 : (_tmp8__label1 = g_quark_from_static_string ("maximize"))))
	do {
		if (window != NULL) {
			wnck_window_maximize (window);
		}
		break;
	} while (0); else if (_tmp8_ == ((0 != _tmp8__label2) ? _tmp8__label2 : (_tmp8__label2 = g_quark_from_static_string ("unmaximize"))))
	do {
		if (window != NULL) {
			wnck_window_unmaximize (window);
		}
		break;
	} while (0); else if (_tmp8_ == ((0 != _tmp8__label3) ? _tmp8__label3 : (_tmp8__label3 = g_quark_from_static_string ("move"))))
	do {
		if (window != NULL) {
			wnck_window_keyboard_move (window);
		}
		break;
	} while (0); else if (_tmp8_ == ((0 != _tmp8__label4) ? _tmp8__label4 : (_tmp8__label4 = g_quark_from_static_string ("resize"))))
	do {
		if (window != NULL) {
			wnck_window_keyboard_size (window);
		}
		break;
	} while (0); else if (_tmp8_ == ((0 != _tmp8__label5) ? _tmp8__label5 : (_tmp8__label5 = g_quark_from_static_string ("topmost"))))
	do {
		if (window != NULL) {
			wnck_window_make_above (window);
		}
		break;
	} while (0); else if (_tmp8_ == ((0 != _tmp8__label6) ? _tmp8__label6 : (_tmp8__label6 = g_quark_from_static_string ("untopmost"))))
	do {
		if (window != NULL) {
			wnck_window_unmake_above (window);
		}
		break;
	} while (0); else if (_tmp8_ == ((0 != _tmp8__label7) ? _tmp8__label7 : (_tmp8__label7 = g_quark_from_static_string ("stick"))))
	do {
		if (window != NULL) {
			wnck_window_stick (window);
		}
		break;
	} while (0); else if (_tmp8_ == ((0 != _tmp8__label8) ? _tmp8__label8 : (_tmp8__label8 = g_quark_from_static_string ("unstick"))))
	do {
		if (window != NULL) {
			wnck_window_unstick (window);
		}
		break;
	} while (0); else if (_tmp8_ == ((0 != _tmp8__label9) ? _tmp8__label9 : (_tmp8__label9 = g_quark_from_static_string ("close"))))
	do {
		if (window != NULL) {
			wnck_window_close (window, gtk_get_current_event_time ());
		}
		break;
		break;
	} while (0); else if (_tmp8_ == ((0 != _tmp8__label10) ? _tmp8__label10 : (_tmp8__label10 = g_quark_from_static_string ("show_desktop"))))
	do {
		GList* windows;
		windows = wnck_screen_get_windows (wnck_screen_get_default ());
		{
			GList* w_collection;
			GList* w_it;
			w_collection = windows;
			for (w_it = w_collection; w_it != NULL; w_it = w_it->next) {
				WnckWindow* w;
				w = (WnckWindow*) w_it->data;
				{
					if (switcher_is_in_sight (self, w)) {
						wnck_window_minimize (w);
					}
				}
			}
		}
		break;
	} while (0); else if (_tmp8_ == ((0 != _tmp8__label11) ? _tmp8__label11 : (_tmp8__label11 = g_quark_from_static_string ("workspaces"))))
	do {
		WorkspaceSelector* ws;
		ws = g_object_ref_sink (workspace_selector_new (window));
		gtk_widget_show ((GtkWidget*) ws);
		(ws == NULL) ? NULL : (ws = (g_object_unref (ws), NULL));
		break;
	} while (0); else
	do {
		if (gtk_menu_item_get_submenu ((GtkMenuItem*) item) != NULL) {
			(window == NULL) ? NULL : (window = (g_object_unref (window), NULL));
			return;
		}
		if (window != NULL) {
			switcher_perhaps_minimize_window (self, window);
		}
		break;
	} while (0);
	(window == NULL) ? NULL : (window = (g_object_unref (window), NULL));
}


/** 
 * Minimize a window if it is not minimized.
 * restore it if minimized.
 *
 */
static void switcher_perhaps_minimize_window (Switcher* self, WnckWindow* window) {
	WnckWorkspace* _tmp0_;
	WnckWorkspace* workspace;
	gboolean _tmp1_;
	WnckScreen* _tmp2_;
	WnckScreen* screen;
	gint current_workspace_x;
	gint current_workspace_y;
	gint x;
	gint y;
	gint w;
	gint h;
	g_return_if_fail (self != NULL);
	g_return_if_fail (window != NULL);
	if (!guess_dock_is_around ()) {
		switcher_set_iconify_destination (self, window);
	}
	_tmp0_ = NULL;
	workspace = (_tmp0_ = wnck_window_get_workspace (window), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	_tmp1_ = FALSE;
	if (wnck_window_is_active (window)) {
		_tmp1_ = wnck_window_is_visible_on_workspace (window, workspace);
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		wnck_window_minimize (window);
		(workspace == NULL) ? NULL : (workspace = (g_object_unref (workspace), NULL));
		return;
	}
	/* Ensure viewport visibility*/
	_tmp2_ = NULL;
	screen = (_tmp2_ = wnck_window_get_screen (window), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_));
	current_workspace_x = wnck_workspace_get_viewport_x (workspace);
	current_workspace_y = wnck_workspace_get_viewport_y (workspace);
	x = 0;
	y = 0;
	w = 0;
	h = 0;
	wnck_window_get_geometry (window, &x, &y, &w, &h);
	wnck_screen_move_viewport (screen, current_workspace_x + x, current_workspace_y + y);
	wnck_window_activate (window, gtk_get_current_event_time ());
	wnck_window_unminimize (window, gtk_get_current_event_time ());
	/* ensure is on top*/
	wnck_window_make_above (window);
	wnck_window_unmake_above (window);
	(workspace == NULL) ? NULL : (workspace = (g_object_unref (workspace), NULL));
	(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
}


static void switcher_set_iconify_destination (Switcher* self, WnckWindow* window) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (window != NULL);
	if (!GTK_WIDGET_REALIZED ((GtkWidget*) self)) {
		return;
	}
	if (wnck_window_get_window_type (window) != WNCK_WINDOW_DESKTOP) {
		gint ax;
		gint ay;
		ax = 0;
		ay = 0;
		gdk_window_get_origin (((GtkWidget*) self)->window, &ax, &ay);
		wnck_window_set_icon_geometry (window, ax, ay, 8, 8);
	}
}


static gboolean switcher_is_in_sight (Switcher* self, WnckWindow* window) {
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (window != NULL, FALSE);
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
	if (wnck_window_is_in_viewport (window, wnck_window_get_workspace (self->priv->_current_window))) {
		_tmp1_ = !wnck_window_is_minimized (window);
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		_tmp0_ = !wnck_window_is_skip_pager (window);
	} else {
		_tmp0_ = FALSE;
	}
	return _tmp0_;
}


static char* switcher_do_xml_menu (Switcher* self) {
	char* items;
	GList* windows;
	char* _tmp22_;
	char* _tmp23_;
	char* _tmp24_;
	g_return_val_if_fail (self != NULL, NULL);
	if (!self->priv->_show_window_list) {
		return g_strdup ("");
	}
	items = g_strdup ("");
	windows = wnck_screen_get_windows (wnck_screen_get_default ());
	{
		GList* window_collection;
		GList* window_it;
		window_collection = windows;
		for (window_it = window_collection; window_it != NULL; window_it = window_it->next) {
			WnckWindow* window;
			window = (WnckWindow*) window_it->data;
			{
				if (!wnck_window_is_skip_pager (window)) {
					char* font;
					char* submenu_xml;
					char** _tmp9_;
					gint substs_size;
					gint substs_length1;
					char** _tmp7_;
					GdkPixbuf* _tmp6_;
					const char* _tmp5_;
					const char* _tmp4_;
					char** _tmp8_;
					char** substs;
					char* item;
					char* _tmp10_;
					font = g_strdup ("");
					submenu_xml = g_strdup ("");
					if (wnck_window_is_active (window)) {
						char* _tmp1_;
						char* _tmp2_;
						_tmp1_ = NULL;
						font = (_tmp1_ = g_strdup ("bold"), font = (g_free (font), NULL), _tmp1_);
						_tmp2_ = NULL;
						submenu_xml = (_tmp2_ = switcher_do_action_menu (self, window), submenu_xml = (g_free (submenu_xml), NULL), _tmp2_);
					} else {
						if (wnck_window_is_minimized (window)) {
							char* _tmp3_;
							_tmp3_ = NULL;
							font = (_tmp3_ = g_strdup ("italic"), font = (g_free (font), NULL), _tmp3_);
						}
					}
					_tmp9_ = NULL;
					_tmp7_ = NULL;
					_tmp6_ = NULL;
					_tmp5_ = NULL;
					_tmp4_ = NULL;
					_tmp8_ = NULL;
					substs = (_tmp9_ = (_tmp8_ = (_tmp7_ = g_new0 (char*, 10 + 1), _tmp7_[0] = g_strdup ("%label%"), _tmp7_[1] = g_markup_escape_text (wnck_window_get_name (window), -1), _tmp7_[2] = g_strdup ("%font%"), _tmp7_[3] = (_tmp4_ = font, (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_)), _tmp7_[4] = g_strdup ("%submenu%"), _tmp7_[5] = (_tmp5_ = submenu_xml, (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_)), _tmp7_[6] = g_strdup ("%pixdata%"), _tmp7_[7] = pixbuf_encode_b64 (_tmp6_ = guess_icon (window, -1, -1)), _tmp7_[8] = g_strdup ("%id%"), _tmp7_[9] = g_strdup_printf ("%lu", wnck_window_get_xid (window)), _tmp7_), (_tmp6_ == NULL) ? NULL : (_tmp6_ = (g_object_unref (_tmp6_), NULL)), _tmp8_), substs_length1 = 10, substs_size = substs_length1, _tmp9_);
					item = template_replace (SWITCHER_ITEM_TEMPLATE, substs, substs_length1);
					_tmp10_ = NULL;
					items = (_tmp10_ = g_strconcat (items, item, NULL), items = (g_free (items), NULL), _tmp10_);
					font = (g_free (font), NULL);
					submenu_xml = (g_free (submenu_xml), NULL);
					substs = (_vala_array_free (substs, substs_length1, (GDestroyNotify) g_free), NULL);
					item = (g_free (item), NULL);
				}
			}
		}
	}
	if (_vala_strcmp0 (items, "") == 0) {
		char* _tmp12_;
		const char* _tmp11_;
		_tmp12_ = NULL;
		_tmp11_ = NULL;
		items = (_tmp12_ = (_tmp11_ = self->priv->NOWIN_TEMPLATE, (_tmp11_ == NULL) ? NULL : g_strdup (_tmp11_)), items = (g_free (items), NULL), _tmp12_);
	} else {
		if (wnck_window_get_window_type (self->priv->_current_window) != WNCK_WINDOW_DESKTOP) {
			char* _tmp13_;
			char* _tmp21_;
			char* _tmp20_;
			char* _tmp19_;
			char* _tmp18_;
			char* _tmp17_;
			char* _tmp16_;
			GdkPixbuf* _tmp15_;
			WnckWindow* _tmp14_;
			_tmp13_ = NULL;
			items = (_tmp13_ = g_strconcat (items, "<item id=\"separator3\" type=\"separator\"/>", NULL), items = (g_free (items), NULL), _tmp13_);
			_tmp21_ = NULL;
			_tmp20_ = NULL;
			_tmp19_ = NULL;
			_tmp18_ = NULL;
			_tmp17_ = NULL;
			_tmp16_ = NULL;
			_tmp15_ = NULL;
			_tmp14_ = NULL;
			items = (_tmp21_ = g_strconcat (items, _tmp20_ = g_strconcat (_tmp19_ = g_strconcat (_tmp18_ = g_strconcat (_tmp17_ = g_strconcat ("<item id=\"show_desktop\" type=\"image\" icon=\"pixbuf:", _tmp16_ = pixbuf_encode_b64 (_tmp15_ = guess_icon (_tmp14_ = find_desktop (), -1, -1)), NULL), "\" label=\"", NULL), _ ("Show _Desktop"), NULL), "\"/>", NULL), NULL), items = (g_free (items), NULL), _tmp21_);
			_tmp20_ = (g_free (_tmp20_), NULL);
			_tmp19_ = (g_free (_tmp19_), NULL);
			_tmp18_ = (g_free (_tmp18_), NULL);
			_tmp17_ = (g_free (_tmp17_), NULL);
			_tmp16_ = (g_free (_tmp16_), NULL);
			(_tmp15_ == NULL) ? NULL : (_tmp15_ = (g_object_unref (_tmp15_), NULL));
			(_tmp14_ == NULL) ? NULL : (_tmp14_ = (g_object_unref (_tmp14_), NULL));
		}
	}
	_tmp22_ = NULL;
	_tmp23_ = NULL;
	_tmp24_ = NULL;
	return (_tmp24_ = (_tmp23_ = g_strconcat (_tmp22_ = g_strconcat ("<menu>", items, NULL), "</menu>", NULL), _tmp22_ = (g_free (_tmp22_), NULL), _tmp23_), items = (g_free (items), NULL), _tmp24_);
}


static char* bool_to_string (gboolean self) {
	if (self) {
		return g_strdup ("true");
	} else {
		return g_strdup ("false");
	}
}


static char* switcher_do_action_menu (Switcher* self, WnckWindow* window) {
	gboolean _tmp3_;
	gboolean _tmp4_;
	gboolean _tmp5_;
	gboolean _tmp6_;
	char** _tmp8_;
	gint substs_size;
	gint substs_length1;
	char** _tmp7_;
	char** substs;
	char* _tmp9_;
	g_return_val_if_fail (self != NULL, NULL);
	if (wnck_window_get_window_type (window) == WNCK_WINDOW_DESKTOP) {
		char* _tmp0_;
		char* _tmp1_;
		_tmp0_ = NULL;
		_tmp1_ = NULL;
		return (_tmp1_ = g_strconcat (_tmp0_ = g_strconcat ("<menu>", self->priv->NOACT_TEMPLATE, NULL), "</menu>", NULL), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
	}
	_tmp3_ = FALSE;
	if ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_MAXIMIZE) != 0) {
		_tmp3_ = !wnck_window_is_maximized (window);
	} else {
		_tmp3_ = FALSE;
	}
	_tmp4_ = FALSE;
	if ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_MAXIMIZE) != 0) {
		_tmp4_ = wnck_window_is_maximized (window);
	} else {
		_tmp4_ = FALSE;
	}
	_tmp5_ = FALSE;
	if ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_ABOVE) != 0) {
		_tmp5_ = wnck_window_is_above (window);
	} else {
		_tmp5_ = FALSE;
	}
	_tmp6_ = FALSE;
	if ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_ABOVE) != 0) {
		_tmp6_ = !wnck_window_is_above (window);
	} else {
		_tmp6_ = FALSE;
	}
	_tmp8_ = NULL;
	_tmp7_ = NULL;
	substs = (_tmp8_ = (_tmp7_ = g_new0 (char*, 24 + 1), _tmp7_[0] = g_strdup ("%visibleminimize%"), _tmp7_[1] = bool_to_string ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_MINIMIZE) != 0), _tmp7_[2] = g_strdup ("%visiblemaximize%"), _tmp7_[3] = bool_to_string (_tmp3_), _tmp7_[4] = g_strdup ("%visibleunmaximize%"), _tmp7_[5] = bool_to_string (_tmp4_), _tmp7_[6] = g_strdup ("%visiblemove%"), _tmp7_[7] = bool_to_string ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_MOVE) != 0), _tmp7_[8] = g_strdup ("%visibleresize"), _tmp7_[9] = bool_to_string ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_RESIZE) != 0), _tmp7_[10] = g_strdup ("%visibleseparator1%"), _tmp7_[11] = bool_to_string ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_ABOVE) != 0), _tmp7_[12] = g_strdup ("%visibleuntopmost%"), _tmp7_[13] = bool_to_string (_tmp5_), _tmp7_[14] = g_strdup ("%visibletopmost%"), _tmp7_[15] = bool_to_string (_tmp6_), _tmp7_[16] = g_strdup ("%visibleunstick%"), _tmp7_[17] = bool_to_string (wnck_window_is_sticky (window)), _tmp7_[18] = g_strdup ("%visiblestick%"), _tmp7_[19] = bool_to_string (!wnck_window_is_sticky (window)), _tmp7_[20] = g_strdup ("%visibleseparator2%"), _tmp7_[21] = bool_to_string ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_CLOSE) != 0), _tmp7_[22] = g_strdup ("%visibleclose%"), _tmp7_[23] = bool_to_string ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_CLOSE) != 0), _tmp7_), substs_length1 = 24, substs_size = substs_length1, _tmp8_);
	_tmp9_ = NULL;
	return (_tmp9_ = template_replace (self->priv->WINACT_TEMPLATE, substs, substs_length1), substs = (_vala_array_free (substs, substs_length1, (GDestroyNotify) g_free), NULL), _tmp9_);
}


static void switcher_update (Switcher* self, gboolean include_menu) {
	GError * _inner_error_;
	gboolean _tmp0_;
	GnomenuApplication* _tmp1_;
	GnomenuApplication* app;
	WnckWindowType wt;
	char* s;
	char* _tmp8_;
	char* _tmp7_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	search_box_menu_item_set_text (self->priv->search_box, "");
	g_object_set ((GtkWidget*) self, "visible", self->priv->_show_icon | self->priv->_show_label, NULL);
	if (!(g_object_get ((GtkWidget*) self, "visible", &_tmp0_, NULL), _tmp0_)) {
		return;
	}
	if (switcher_get_current_window (self) == NULL) {
		return;
	}
	_tmp1_ = NULL;
	app = (_tmp1_ = gnomenu_application_lookup_from_wnck (wnck_window_get_application (switcher_get_current_window (self))), (_tmp1_ == NULL) ? NULL : gnomenu_application_ref (_tmp1_));
	wt = wnck_window_get_window_type (switcher_get_current_window (self));
	if (wt == WNCK_WINDOW_DOCK) {
		char* _tmp2_;
		_tmp2_ = NULL;
		self->priv->_label = (_tmp2_ = g_strdup (""), self->priv->_label = (g_free (self->priv->_label), NULL), _tmp2_);
	} else {
		if (app != NULL) {
			const char* _tmp3_;
			char* _tmp5_;
			const char* _tmp4_;
			_tmp3_ = NULL;
			if (wnck_window_get_window_type (switcher_get_current_window (self)) == WNCK_WINDOW_DESKTOP) {
				_tmp3_ = _ ("Desktop");
			} else {
				_tmp3_ = gnomenu_application_get_readable_name (app);
			}
			_tmp5_ = NULL;
			_tmp4_ = NULL;
			self->priv->_label = (_tmp5_ = (_tmp4_ = _tmp3_, (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_)), self->priv->_label = (g_free (self->priv->_label), NULL), _tmp5_);
		} else {
			char* _tmp6_;
			_tmp6_ = NULL;
			self->priv->_label = (_tmp6_ = g_strdup ("app unknown shouldn't see this"), self->priv->_label = (g_free (self->priv->_label), NULL), _tmp6_);
		}
	}
	s = g_strdup (SWITCHER_MENU_TEMPLATE);
	_tmp8_ = NULL;
	_tmp7_ = NULL;
	s = (_tmp8_ = replace (s, "%label%", _tmp7_ = g_markup_escape_text (self->priv->_label, -1)), s = (g_free (s), NULL), _tmp8_);
	_tmp7_ = (g_free (_tmp7_), NULL);
	if (self->priv->_show_icon) {
		gint width;
		gint height;
		char* _tmp11_;
		if (self->priv->_show_label) {
			char* _tmp9_;
			_tmp9_ = NULL;
			s = (_tmp9_ = replace (s, "%type%", "image"), s = (g_free (s), NULL), _tmp9_);
		} else {
			char* _tmp10_;
			_tmp10_ = NULL;
			s = (_tmp10_ = replace (s, "%type%", "icon"), s = (g_free (s), NULL), _tmp10_);
		}
		width = ((GtkWidget*) self)->allocation.width - 2;
		height = ((GtkWidget*) self)->allocation.height - 2;
		if (width <= 0) {
			width = -1;
		}
		if (height <= 0) {
			height = -1;
		}
		_tmp11_ = NULL;
		g_debug ("switcher.vala:310: iconname = %s no_in_menu = %s", gnomenu_application_get_icon_name (app), _tmp11_ = bool_to_string (gnomenu_application_get_not_in_menu (app)));
		_tmp11_ = (g_free (_tmp11_), NULL);
		if (gnomenu_application_get_icon_pixbuf (app) != NULL) {
			char* _tmp14_;
			char* _tmp13_;
			char* _tmp12_;
			_tmp14_ = NULL;
			_tmp13_ = NULL;
			_tmp12_ = NULL;
			s = (_tmp14_ = replace (s, "%icon%", _tmp13_ = g_strconcat ("pixbuf:", _tmp12_ = pixbuf_encode_b64 (gnomenu_application_get_icon_pixbuf (app)), NULL)), s = (g_free (s), NULL), _tmp14_);
			_tmp13_ = (g_free (_tmp13_), NULL);
			_tmp12_ = (g_free (_tmp12_), NULL);
		} else {
			char* _tmp16_;
			char* _tmp15_;
			_tmp16_ = NULL;
			_tmp15_ = NULL;
			s = (_tmp16_ = replace (s, "%icon%", _tmp15_ = g_strconcat ("theme:", gnomenu_application_get_icon_name (app), NULL)), s = (g_free (s), NULL), _tmp16_);
			_tmp15_ = (g_free (_tmp15_), NULL);
		}
	} else {
		char* _tmp17_;
		char* _tmp18_;
		_tmp17_ = NULL;
		s = (_tmp17_ = replace (s, "%type%", "normal"), s = (g_free (s), NULL), _tmp17_);
		_tmp18_ = NULL;
		s = (_tmp18_ = replace (s, "icon=\"%icon%\"", ""), s = (g_free (s), NULL), _tmp18_);
	}
	if (include_menu) {
		if (self->priv->_show_window_list) {
			char* _tmp20_;
			char* _tmp19_;
			GtkMenu* _tmp21_;
			GtkMenu* menu;
			_tmp20_ = NULL;
			_tmp19_ = NULL;
			s = (_tmp20_ = replace (s, "%submenu%", _tmp19_ = switcher_do_xml_menu (self)), s = (g_free (s), NULL), _tmp20_);
			_tmp19_ = (g_free (_tmp19_), NULL);
			gnomenu_parser_parse ((GnomenuShell*) self, s, &_inner_error_);
			if (_inner_error_ != NULL) {
				(app == NULL) ? NULL : (app = (gnomenu_application_unref (app), NULL));
				s = (g_free (s), NULL);
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return;
			}
			_tmp21_ = NULL;
			menu = (_tmp21_ = gtk_menu_item_get_submenu ((GtkMenuItem*) self->priv->_label_item), (_tmp21_ == NULL) ? NULL : g_object_ref (_tmp21_));
			/*Gnomenu.MenuItem misd = this.get("/switcher/show_desktop");
			if (misd!=null) {
			override_item_window(misd, find_desktop());
			}*/
			{
				GList* widget_collection;
				GList* widget_it;
				widget_collection = gtk_container_get_children ((GtkContainer*) menu);
				for (widget_it = widget_collection; widget_it != NULL; widget_it = widget_it->next) {
					GtkWidget* _tmp24_;
					GtkWidget* widget;
					_tmp24_ = NULL;
					widget = (_tmp24_ = (GtkWidget*) widget_it->data, (_tmp24_ == NULL) ? NULL : g_object_ref (_tmp24_));
					{
						GnomenuMenuItem* _tmp23_;
						GtkWidget* _tmp22_;
						GnomenuMenuItem* item;
						WnckWindow* window;
						_tmp23_ = NULL;
						_tmp22_ = NULL;
						item = (_tmp23_ = (_tmp22_ = widget, GNOMENU_IS_MENU_ITEM (_tmp22_) ? ((GnomenuMenuItem*) _tmp22_) : NULL), (_tmp23_ == NULL) ? NULL : g_object_ref (_tmp23_));
						if (item == NULL) {
							(widget == NULL) ? NULL : (widget = (g_object_unref (widget), NULL));
							(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
							continue;
						}
						gnomenu_menu_item_set_max_width_chars (item, 30);
						/* Some sane value?
						libgnomenu's default
						is 30*/
						window = switcher_item_to_window (self, item);
						if (window == NULL) {
							(widget == NULL) ? NULL : (widget = (g_object_unref (widget), NULL));
							(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
							(window == NULL) ? NULL : (window = (g_object_unref (window), NULL));
							continue;
						}
						(widget == NULL) ? NULL : (widget = (g_object_unref (widget), NULL));
						(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
						(window == NULL) ? NULL : (window = (g_object_unref (window), NULL));
					}
				}
				(widget_collection == NULL) ? NULL : (widget_collection = (g_list_free (widget_collection), NULL));
			}
			(menu == NULL) ? NULL : (menu = (g_object_unref (menu), NULL));
		} else {
			/*if (window.is_active()) {
			setup_window_actions_menu(
			"/switcher/" + item.item_id + "/", 
			window);
			}*/
			if (self->priv->_show_window_actions) {
				char* _tmp26_;
				char* _tmp25_;
				_tmp26_ = NULL;
				_tmp25_ = NULL;
				s = (_tmp26_ = replace (s, "%submenu%", _tmp25_ = switcher_do_action_menu (self, self->priv->_current_window)), s = (g_free (s), NULL), _tmp26_);
				_tmp25_ = (g_free (_tmp25_), NULL);
				gnomenu_parser_parse ((GnomenuShell*) self, s, &_inner_error_);
				if (_inner_error_ != NULL) {
					(app == NULL) ? NULL : (app = (gnomenu_application_unref (app), NULL));
					s = (g_free (s), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return;
				}
			}
		}
	} else {
		char* _tmp27_;
		/*setup_window_actions_menu("/switcher/", _current_window);*/
		_tmp27_ = NULL;
		s = (_tmp27_ = replace (s, "%submenu%", "<menu/>"), s = (g_free (s), NULL), _tmp27_);
		gnomenu_parser_parse ((GnomenuShell*) self, s, &_inner_error_);
		if (_inner_error_ != NULL) {
			(app == NULL) ? NULL : (app = (gnomenu_application_unref (app), NULL));
			s = (g_free (s), NULL);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	(app == NULL) ? NULL : (app = (gnomenu_application_unref (app), NULL));
	s = (g_free (s), NULL);
}


/*private void setup_window_actions_menu(string prefix, Wnck.Window window) {
string[] actions = {"minimize",
"maximize",
"unmaximize",
"move",
"resize",
"topmost",
"untopmost",
"stick",
"unstick",
"workspaces",
"close"};
for (int co=0; co<actions.length; co++) {
Gnomenu.MenuItem si = this.get(prefix +actions[co]);
if(si != null) {
override_item_window(si, window);
}
}
}
*
 * The following 3 functions map item to window and verse vesa
 *
 */
static GnomenuMenuItem* switcher_window_to_item (Switcher* self, WnckWindow* window) {
	char* _tmp1_;
	char* _tmp0_;
	GnomenuMenuItem* _tmp2_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (window != NULL, NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	_tmp2_ = NULL;
	return (_tmp2_ = gnomenu_menu_bar_get ((GnomenuMenuBar*) self, _tmp1_ = g_strconcat ("/switcher/XID", _tmp0_ = g_strdup_printf ("%lu", wnck_window_get_xid (window)), NULL)), _tmp1_ = (g_free (_tmp1_), NULL), _tmp0_ = (g_free (_tmp0_), NULL), _tmp2_);
}


/*private void override_item_window(Gnomenu.MenuItem item, Wnck.Window? window) {
item.user_data = window;
}*/
static WnckWindow* switcher_item_to_window (Switcher* self, GnomenuMenuItem* item) {
	const char* _tmp0_;
	char* id;
	WnckWindow* _tmp3_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (item != NULL, NULL);
	_tmp0_ = NULL;
	id = (_tmp0_ = gnomenu_item_get_item_id ((GnomenuItem*) item), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	/*if(item.user_data != null) return item.user_data as Wnck.Window;*/
	if (g_str_has_prefix (id, "XID")) {
		WnckWindow* _tmp1_;
		WnckWindow* _tmp2_;
		_tmp1_ = NULL;
		_tmp2_ = NULL;
		return (_tmp2_ = (_tmp1_ = wnck_window_get (strtoul (g_utf8_offset_to_pointer (id, (glong) 3), NULL, 0)), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), id = (g_free (id), NULL), _tmp2_);
	}
	_tmp3_ = NULL;
	return (_tmp3_ = NULL, id = (g_free (id), NULL), _tmp3_);
}


static void switcher_real_size_request (GtkWidget* base, GtkRequisition* req) {
	Switcher * self;
	gboolean _tmp0_;
	self = (Switcher*) base;
	GTK_WIDGET_CLASS (switcher_parent_class)->size_request ((GtkWidget*) GNOMENU_MENU_BAR (self), &(*req));
	_tmp0_ = FALSE;
	if (switcher_get_max_size (self) > 0) {
		_tmp0_ = (*req).width > switcher_get_max_size (self);
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		/* if max_size == -1, we don't truncate it*/
		(*req).width = switcher_get_max_size (self);
	}
}


WnckWindow* switcher_get_current_window (Switcher* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_current_window;
}


void switcher_set_current_window (Switcher* self, WnckWindow* value) {
	WnckWindow* _tmp2_;
	WnckWindow* _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp2_ = NULL;
	_tmp1_ = NULL;
	self->priv->_current_window = (_tmp2_ = (_tmp1_ = value, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (self->priv->_current_window == NULL) ? NULL : (self->priv->_current_window = (g_object_unref (self->priv->_current_window), NULL)), _tmp2_);
	if (!guess_dock_is_around ()) {
		switcher_set_iconify_destination (self, self->priv->_current_window);
	}
	/* always refresh !*/
	switcher_update (self, TRUE);
	g_object_notify ((GObject *) self, "current-window");
}


gint switcher_get_max_size (Switcher* self) {
	g_return_val_if_fail (self != NULL, 0);
	return self->priv->_max_size;
}


void switcher_set_max_size (Switcher* self, gint value) {
	g_return_if_fail (self != NULL);
	if (self->priv->_max_size == value) {
		return;
	}
	self->priv->_max_size = value;
	if (self->priv->_max_size < 40) {
		self->priv->_max_size = -1;
	}
	switcher_update (self, TRUE);
	g_object_notify ((GObject *) self, "max-size");
}


gboolean switcher_get_show_icon (Switcher* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_show_icon;
}


void switcher_set_show_icon (Switcher* self, gboolean value) {
	g_return_if_fail (self != NULL);
	if (self->priv->_show_icon == value) {
		return;
	}
	self->priv->_show_icon = value;
	switcher_update (self, TRUE);
	g_object_notify ((GObject *) self, "show-icon");
}


gboolean switcher_get_show_label (Switcher* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_show_label;
}


void switcher_set_show_label (Switcher* self, gboolean value) {
	g_return_if_fail (self != NULL);
	if (self->priv->_show_label == value) {
		return;
	}
	self->priv->_show_label = value;
	switcher_update (self, TRUE);
	g_object_notify ((GObject *) self, "show-label");
}


gboolean switcher_get_show_window_list (Switcher* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_show_window_list;
}


void switcher_set_show_window_list (Switcher* self, gboolean value) {
	g_return_if_fail (self != NULL);
	if (self->priv->_show_window_list == value) {
		return;
	}
	self->priv->_show_window_list = value;
	switcher_update (self, TRUE);
	g_object_notify ((GObject *) self, "show-window-list");
}


gboolean switcher_get_show_window_actions (Switcher* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_show_window_actions;
}


void switcher_set_show_window_actions (Switcher* self, gboolean value) {
	g_return_if_fail (self != NULL);
	if (self->priv->_show_window_actions == value) {
		return;
	}
	self->priv->_show_window_actions = value;
	switcher_update (self, TRUE);
	g_object_notify ((GObject *) self, "show-window-actions");
}


static void _switcher_on_activate_gnomenu_menu_bar_activate (Switcher* _sender, GnomenuMenuItem* item, gpointer self) {
	switcher_on_activate (self, _sender, item);
}


static void _switcher_on_deactivate_gtk_menu_shell_deactivate (Switcher* _sender, gpointer self) {
	switcher_on_deactivate (self);
}


static GObject * switcher_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	SwitcherClass * klass;
	GObjectClass * parent_class;
	Switcher * self;
	GError * _inner_error_;
	klass = SWITCHER_CLASS (g_type_class_peek (TYPE_SWITCHER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = SWITCHER (obj);
	_inner_error_ = NULL;
	{
		char* _tmp3_;
		char* _tmp4_;
		char** _tmp17_;
		gint substs_size;
		gint substs_length1;
		char** _tmp16_;
		const char* _tmp15_;
		const char* _tmp14_;
		const char* _tmp13_;
		const char* _tmp12_;
		const char* _tmp11_;
		const char* _tmp10_;
		const char* _tmp9_;
		const char* _tmp8_;
		const char* _tmp7_;
		const char* _tmp6_;
		const char* _tmp5_;
		char** substs;
		char* _tmp18_;
		{
			char* s;
			char* _tmp0_;
			GnomenuMenuItem* _tmp2_;
			GnomenuItem* _tmp1_;
			s = replace (SWITCHER_MENU_TEMPLATE, "%label%", "Global Menu Bar");
			_tmp0_ = NULL;
			s = (_tmp0_ = replace (s, "%sub-menu%", ""), s = (g_free (s), NULL), _tmp0_);
			gnomenu_parser_parse ((GnomenuShell*) self, s, &_inner_error_);
			if (_inner_error_ != NULL) {
				s = (g_free (s), NULL);
				goto __catch9_g_error;
				goto __finally9;
			}
			_tmp2_ = NULL;
			_tmp1_ = NULL;
			self->priv->_label_item = (_tmp2_ = (_tmp1_ = gnomenu_shell_get_item ((GnomenuShell*) self, 0), GNOMENU_IS_MENU_ITEM (_tmp1_) ? ((GnomenuMenuItem*) _tmp1_) : NULL), (self->priv->_label_item == NULL) ? NULL : (self->priv->_label_item = (g_object_unref (self->priv->_label_item), NULL)), _tmp2_);
			gtk_menu_shell_append ((GtkMenuShell*) self->priv->_label_item->_submenu_cache, (GtkWidget*) ((GtkMenuItem*) self->priv->search_box));
			g_signal_connect_object ((GnomenuMenuBar*) self, "activate", (GCallback) _switcher_on_activate_gnomenu_menu_bar_activate, self, 0);
			s = (g_free (s), NULL);
		}
		goto __finally9;
		__catch9_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
				g_warning ("switcher.vala:65: %s", e->message);
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally9:
		if (_inner_error_ != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
		}
		_tmp3_ = NULL;
		self->priv->NOWIN_TEMPLATE = (_tmp3_ = template_replace_simple (self->priv->NOWIN_TEMPLATE, "%no_windows%", _ ("no windows")), self->priv->NOWIN_TEMPLATE = (g_free (self->priv->NOWIN_TEMPLATE), NULL), _tmp3_);
		_tmp4_ = NULL;
		self->priv->NOACT_TEMPLATE = (_tmp4_ = template_replace_simple (self->priv->NOACT_TEMPLATE, "%no_actions%", _ ("no actions")), self->priv->NOACT_TEMPLATE = (g_free (self->priv->NOACT_TEMPLATE), NULL), _tmp4_);
		_tmp17_ = NULL;
		_tmp16_ = NULL;
		_tmp15_ = NULL;
		_tmp14_ = NULL;
		_tmp13_ = NULL;
		_tmp12_ = NULL;
		_tmp11_ = NULL;
		_tmp10_ = NULL;
		_tmp9_ = NULL;
		_tmp8_ = NULL;
		_tmp7_ = NULL;
		_tmp6_ = NULL;
		_tmp5_ = NULL;
		substs = (_tmp17_ = (_tmp16_ = g_new0 (char*, 46 + 1), _tmp16_[0] = g_strdup ("%labelminimize%"), _tmp16_[1] = (_tmp5_ = _ ("Mi_nimize"), (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_)), _tmp16_[2] = g_strdup ("%visiblemaximize%"), _tmp16_[3] = g_strdup ("%visiblemaximize%"), _tmp16_[4] = g_strdup ("%labelmaximize%"), _tmp16_[5] = (_tmp6_ = _ ("Ma_ximize"), (_tmp6_ == NULL) ? NULL : g_strdup (_tmp6_)), _tmp16_[6] = g_strdup ("%visibleminimize%"), _tmp16_[7] = g_strdup ("%visibleminimize%"), _tmp16_[8] = g_strdup ("%labelunmaximize%"), _tmp16_[9] = (_tmp7_ = _ ("Unma_ximize"), (_tmp7_ == NULL) ? NULL : g_strdup (_tmp7_)), _tmp16_[10] = g_strdup ("%visibleunmaximize%"), _tmp16_[11] = g_strdup ("%visibleunmaximize%"), _tmp16_[12] = g_strdup ("%labelmove%"), _tmp16_[13] = (_tmp8_ = _ ("_Move"), (_tmp8_ == NULL) ? NULL : g_strdup (_tmp8_)), _tmp16_[14] = g_strdup ("%visiblemove%"), _tmp16_[15] = g_strdup ("%visiblemove%"), _tmp16_[16] = g_strdup ("%labelresize%"), _tmp16_[17] = (_tmp9_ = _ ("_Resize"), (_tmp9_ == NULL) ? NULL : g_strdup (_tmp9_)), _tmp16_[18] = g_strdup ("%visibleresize"), _tmp16_[19] = g_strdup ("%visibleresize%"), _tmp16_[20] = g_strdup ("%visibleseparator1%"), _tmp16_[21] = g_strdup ("%visibleseparator1%"), _tmp16_[22] = g_strdup ("%labeluntopmost%"), _tmp16_[23] = (_tmp10_ = _ ("Always on _Top"), (_tmp10_ == NULL) ? NULL : g_strdup (_tmp10_)), _tmp16_[24] = g_strdup ("%visibleuntopmost%"), _tmp16_[25] = g_strdup ("%visibleuntopmost%"), _tmp16_[26] = g_strdup ("%labeltopmost%"), _tmp16_[27] = (_tmp11_ = _ ("Always on _Top"), (_tmp11_ == NULL) ? NULL : g_strdup (_tmp11_)), _tmp16_[28] = g_strdup ("%visibletopmost%"), _tmp16_[29] = g_strdup ("%visibletopmost%"), _tmp16_[30] = g_strdup ("%labelunstick%"), _tmp16_[31] = (_tmp12_ = _ ("_Always on visible Workspace"), (_tmp12_ == NULL) ? NULL : g_strdup (_tmp12_)), _tmp16_[32] = g_strdup ("%visibleunstick%"), _tmp16_[33] = g_strdup ("%visibleunstick%"), _tmp16_[34] = g_strdup ("%labelstick%"), _tmp16_[35] = (_tmp13_ = _ ("_Always on visible Workspace"), (_tmp13_ == NULL) ? NULL : g_strdup (_tmp13_)), _tmp16_[36] = g_strdup ("%visiblestick%"), _tmp16_[37] = g_strdup ("%visiblestick%"), _tmp16_[38] = g_strdup ("%labelworkspaces%"), _tmp16_[39] = (_tmp14_ = _ ("_Workspace..."), (_tmp14_ == NULL) ? NULL : g_strdup (_tmp14_)), _tmp16_[40] = g_strdup ("%visibleseparator2%"), _tmp16_[41] = g_strdup ("%visibleseparator2%"), _tmp16_[42] = g_strdup ("%labelclose%"), _tmp16_[43] = (_tmp15_ = _ ("_Close"), (_tmp15_ == NULL) ? NULL : g_strdup (_tmp15_)), _tmp16_[44] = g_strdup ("%visibleclose%"), _tmp16_[45] = g_strdup ("%visibleclose%"), _tmp16_), substs_length1 = 46, substs_size = substs_length1, _tmp17_);
		_tmp18_ = NULL;
		self->priv->WINACT_TEMPLATE = (_tmp18_ = template_replace (self->priv->WINACT_TEMPLATE, substs, substs_length1), self->priv->WINACT_TEMPLATE = (g_free (self->priv->WINACT_TEMPLATE), NULL), _tmp18_);
		/* fixes a rendering issue with Compiz */
		g_signal_connect_object ((GtkMenuShell*) self, "deactivate", (GCallback) _switcher_on_deactivate_gtk_menu_shell_deactivate, self, 0);
		substs = (_vala_array_free (substs, substs_length1, (GDestroyNotify) g_free), NULL);
	}
	return obj;
}


static void switcher_class_init (SwitcherClass * klass) {
	switcher_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (SwitcherPrivate));
	G_OBJECT_CLASS (klass)->dispose = switcher_real_dispose;
	GTK_WIDGET_CLASS (klass)->size_request = switcher_real_size_request;
	G_OBJECT_CLASS (klass)->get_property = switcher_get_property;
	G_OBJECT_CLASS (klass)->set_property = switcher_set_property;
	G_OBJECT_CLASS (klass)->constructor = switcher_constructor;
	G_OBJECT_CLASS (klass)->finalize = switcher_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), SWITCHER_CURRENT_WINDOW, g_param_spec_object ("current-window", "current-window", "current-window", WNCK_TYPE_WINDOW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), SWITCHER_MAX_SIZE, g_param_spec_int ("max-size", "max-size", "max-size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), SWITCHER_SHOW_ICON, g_param_spec_boolean ("show-icon", "show-icon", "show-icon", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), SWITCHER_SHOW_LABEL, g_param_spec_boolean ("show-label", "show-label", "show-label", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), SWITCHER_SHOW_WINDOW_LIST, g_param_spec_boolean ("show-window-list", "show-window-list", "show-window-list", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), SWITCHER_SHOW_WINDOW_ACTIONS, g_param_spec_boolean ("show-window-actions", "show-window-actions", "show-window-actions", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	{
		GType type;
		GTypeClass* _tmp19_;
		/* To load wnck-stock-delete
		 * wnck-stock-maximize
		 * wnck-stock-minimize
		 * */
		type = WNCK_TYPE_ACTION_MENU;
		_tmp19_ = NULL;
		_tmp19_ = g_type_class_ref (type);
		(_tmp19_ == NULL) ? NULL : (_tmp19_ = (g_type_class_unref (_tmp19_), NULL));
	}
}


static void switcher_instance_init (Switcher * self) {
	self->priv = SWITCHER_GET_PRIVATE (self);
	self->priv->_max_size = -1;
	self->priv->_show_icon = FALSE;
	self->priv->_show_label = TRUE;
	self->priv->_show_window_list = TRUE;
	self->priv->_show_window_actions = TRUE;
	self->priv->NOWIN_TEMPLATE = g_strdup ("<item label=\"%no_windows%\" type=\"image\" icon=\"theme:gtk-about\" sensitive=\"false\" font=\"italic\"/>");
	self->priv->NOACT_TEMPLATE = g_strdup ("<item label=\"%no_actions%\" type=\"image\" icon=\"theme:gtk-about\" sensitive=\"false\" font=\"italic\"/>");
	self->priv->WINACT_TEMPLATE = g_strdup ("<menu>\n\t<item id=\"maximize\" type=\"image\" icon=\"wnck-stock-maximize\" label=\"%labelmaximize%\" visible=\"%visiblemaximize%\" />\n\t<item id=\"minimize\" type=\"image\" icon=\"wnck-stock-minimize\" label=\"%labelminimize%\" visible=\"%visibleminimize%\" />\n\t<item id=\"unmaximize\" type=\"normal\" label=\"%labelunmaximize%\" visible=\"%visibleunmaximize%\" />\n\t<item id=\"move\" type=\"normal\" label=\"%labelmove%\" visible=\"%visiblemove%\" />\n\t<item id=\"resize\" type=\"normal\" label=\"%labelresize%\" visible=\"%visibleresize%\" />\n\t<item id=\"separator1\" type=\"separator\" visible=\"%visibleseparator1%\" />\n\t<item id=\"untopmost\" type=\"check\" state=\"toggled\" label=\"%labeluntopmost%\" visible=\"%visibleuntopmost%\" />\n\t<item id=\"topmost\" type=\"check\" state=\"untoggled\" label=\"%labeltopmost%\" visible=\"%visibletopmost%\" />\n\t<item id=\"unstick\" type=\"check\" state=\"toggled\" label=\"%labelunstick%\" visible=\"%visibleunstick%\" />\n\t<item id=\"stick\" type=\"check\" state=\"untoggled\" label=\"%labelstick%\" visible=\"%visiblestick%\" />\n\t<item id=\"workspaces\" type=\"normal\" label=\"%labelworkspaces%\" />\n\t<item id=\"separator2\" type=\"separator\" visible=\"%visibleseparator2%\" />\n\t<item id=\"close\" type=\"image\" icon=\"wnck-stock-delete\" label=\"%labelclose%\" visible=\"%visibleclose%\" />\n</menu>");
	self->priv->search_box = g_object_ref_sink (search_box_menu_item_new ());
	self->priv->disposed = FALSE;
}


static void switcher_finalize (GObject* obj) {
	Switcher * self;
	self = SWITCHER (obj);
	self->priv->_label = (g_free (self->priv->_label), NULL);
	(self->priv->_label_item == NULL) ? NULL : (self->priv->_label_item = (g_object_unref (self->priv->_label_item), NULL));
	(self->priv->_current_window == NULL) ? NULL : (self->priv->_current_window = (g_object_unref (self->priv->_current_window), NULL));
	self->priv->NOWIN_TEMPLATE = (g_free (self->priv->NOWIN_TEMPLATE), NULL);
	self->priv->NOACT_TEMPLATE = (g_free (self->priv->NOACT_TEMPLATE), NULL);
	self->priv->WINACT_TEMPLATE = (g_free (self->priv->WINACT_TEMPLATE), NULL);
	(self->priv->search_box == NULL) ? NULL : (self->priv->search_box = (g_object_unref (self->priv->search_box), NULL));
	G_OBJECT_CLASS (switcher_parent_class)->finalize (obj);
}


GType switcher_get_type (void) {
	static GType switcher_type_id = 0;
	if (switcher_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (SwitcherClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) switcher_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Switcher), 0, (GInstanceInitFunc) switcher_instance_init, NULL };
		switcher_type_id = g_type_register_static (GNOMENU_TYPE_MENU_BAR, "Switcher", &g_define_type_info, 0);
	}
	return switcher_type_id;
}


static void switcher_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	Switcher * self;
	gpointer boxed;
	self = SWITCHER (object);
	switch (property_id) {
		case SWITCHER_CURRENT_WINDOW:
		g_value_set_object (value, switcher_get_current_window (self));
		break;
		case SWITCHER_MAX_SIZE:
		g_value_set_int (value, switcher_get_max_size (self));
		break;
		case SWITCHER_SHOW_ICON:
		g_value_set_boolean (value, switcher_get_show_icon (self));
		break;
		case SWITCHER_SHOW_LABEL:
		g_value_set_boolean (value, switcher_get_show_label (self));
		break;
		case SWITCHER_SHOW_WINDOW_LIST:
		g_value_set_boolean (value, switcher_get_show_window_list (self));
		break;
		case SWITCHER_SHOW_WINDOW_ACTIONS:
		g_value_set_boolean (value, switcher_get_show_window_actions (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void switcher_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	Switcher * self;
	self = SWITCHER (object);
	switch (property_id) {
		case SWITCHER_CURRENT_WINDOW:
		switcher_set_current_window (self, g_value_get_object (value));
		break;
		case SWITCHER_MAX_SIZE:
		switcher_set_max_size (self, g_value_get_int (value));
		break;
		case SWITCHER_SHOW_ICON:
		switcher_set_show_icon (self, g_value_get_boolean (value));
		break;
		case SWITCHER_SHOW_LABEL:
		switcher_set_show_label (self, g_value_get_boolean (value));
		break;
		case SWITCHER_SHOW_WINDOW_LIST:
		switcher_set_show_window_list (self, g_value_get_boolean (value));
		break;
		case SWITCHER_SHOW_WINDOW_ACTIONS:
		switcher_set_show_window_actions (self, g_value_get_boolean (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


SearchBoxMenuItem* search_box_menu_item_construct (GType object_type) {
	SearchBoxMenuItem * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


SearchBoxMenuItem* search_box_menu_item_new (void) {
	return search_box_menu_item_construct (TYPE_SEARCH_BOX_MENU_ITEM);
}


GnomenuMenuItem* search_box_menu_item_find_item (SearchBoxMenuItem* self) {
	GnomenuShell* _tmp1_;
	GtkContainer* _tmp0_;
	GnomenuShell* shell;
	GnomenuMenuItem* _tmp9_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	shell = (_tmp1_ = (_tmp0_ = gtk_widget_get_parent ((GtkWidget*) self), GNOMENU_IS_SHELL (_tmp0_) ? ((GnomenuShell*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_));
	{
		gint i;
		i = 0;
		for (; i < gnomenu_shell_get_length (shell); i++) {
			GnomenuItem* _tmp2_;
			GnomenuMenuItem* item;
			gboolean _tmp3_;
			_tmp2_ = NULL;
			item = (_tmp2_ = gnomenu_shell_get_item (shell, i), GNOMENU_IS_MENU_ITEM (_tmp2_) ? ((GnomenuMenuItem*) _tmp2_) : NULL);
			_tmp3_ = FALSE;
			if (gnomenu_item_get_item_label ((GnomenuItem*) item) != NULL) {
				char* _tmp5_;
				char* _tmp4_;
				_tmp5_ = NULL;
				_tmp4_ = NULL;
				_tmp3_ = g_str_has_prefix (_tmp4_ = g_utf8_casefold (gnomenu_item_get_item_label ((GnomenuItem*) item), -1), _tmp5_ = g_utf8_casefold (gtk_entry_get_text (self->priv->textbox), -1));
				_tmp5_ = (g_free (_tmp5_), NULL);
				_tmp4_ = (g_free (_tmp4_), NULL);
			} else {
				_tmp3_ = FALSE;
			}
			if (_tmp3_) {
				GnomenuMenuItem* _tmp7_;
				GnomenuMenuItem* _tmp6_;
				GnomenuMenuItem* _tmp8_;
				_tmp7_ = NULL;
				_tmp6_ = NULL;
				_tmp8_ = NULL;
				return (_tmp8_ = (_tmp7_ = (_tmp6_ = item, GNOMENU_IS_MENU_ITEM (_tmp6_) ? ((GnomenuMenuItem*) _tmp6_) : NULL), (_tmp7_ == NULL) ? NULL : g_object_ref (_tmp7_)), (item == NULL) ? NULL : (item = (g_object_unref (item), NULL)), (shell == NULL) ? NULL : (shell = (g_object_unref (shell), NULL)), _tmp8_);
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
	}
	_tmp9_ = NULL;
	return (_tmp9_ = NULL, (shell == NULL) ? NULL : (shell = (g_object_unref (shell), NULL)), _tmp9_);
}


static gboolean _search_box_menu_item_steal_key_press_event_gtk_widget_key_press_event (GtkWidget* _sender, const GdkEventKey* event, gpointer self) {
	return search_box_menu_item_steal_key_press_event (self, _sender, event);
}


static void search_box_menu_item_real_parent_set (GtkWidget* base, GtkWidget* previous_parent) {
	SearchBoxMenuItem * self;
	self = (SearchBoxMenuItem*) base;
	if (previous_parent != NULL) {
		guint _tmp0_;
		g_signal_handlers_disconnect_matched (previous_parent, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("key-press-event", GTK_TYPE_WIDGET, &_tmp0_, NULL, FALSE), _tmp0_), 0, NULL, (GCallback) _search_box_menu_item_steal_key_press_event_gtk_widget_key_press_event, self);
	}
	GTK_WIDGET_CLASS (search_box_menu_item_parent_class)->parent_set ((GtkWidget*) GTK_IMAGE_MENU_ITEM (self), previous_parent);
	if (gtk_widget_get_parent ((GtkWidget*) self) != NULL) {
		g_signal_connect_object ((GtkWidget*) gtk_widget_get_parent ((GtkWidget*) self), "key-press-event", (GCallback) _search_box_menu_item_steal_key_press_event_gtk_widget_key_press_event, self, 0);
	}
}


static gboolean search_box_menu_item_steal_key_press_event (SearchBoxMenuItem* self, GtkWidget* widget, const GdkEventKey* event) {
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (widget != NULL, FALSE);
	return (g_signal_emit_by_name ((GtkWidget*) self->priv->textbox, "key-press-event", &(*event), &_tmp0_), _tmp0_);
}


static void search_box_menu_item_real_activate_item (GtkMenuItem* base) {
	SearchBoxMenuItem * self;
	self = (SearchBoxMenuItem*) base;
}


static void search_box_menu_item_real_activate (GtkMenuItem* base) {
	SearchBoxMenuItem * self;
	self = (SearchBoxMenuItem*) base;
}


const char* search_box_menu_item_get_text (SearchBoxMenuItem* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return gtk_entry_get_text (self->priv->textbox);
}


void search_box_menu_item_set_text (SearchBoxMenuItem* self, const char* value) {
	g_return_if_fail (self != NULL);
	gtk_entry_set_text (self->priv->textbox, value);
	g_object_notify ((GObject *) self, "text");
}


static void _lambda0_ (GtkEntry* box, GtkStateType previous_state, SearchBoxMenuItem* self) {
	g_return_if_fail (box != NULL);
	((GtkWidget*) self->priv->textbox)->state = (guchar) GTK_STATE_NORMAL;
}


static void __lambda0__gtk_widget_state_changed (GtkEntry* _sender, GtkStateType previous_state, gpointer self) {
	_lambda0_ (_sender, previous_state, self);
}


static void _lambda1_ (GtkEntry* box, SearchBoxMenuItem* self) {
	GnomenuMenu* _tmp1_;
	GtkContainer* _tmp0_;
	GnomenuMenu* shell;
	GnomenuMenuItem* item;
	g_return_if_fail (box != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	shell = (_tmp1_ = (_tmp0_ = gtk_widget_get_parent ((GtkWidget*) self), GNOMENU_IS_MENU (_tmp0_) ? ((GnomenuMenu*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_));
	item = search_box_menu_item_find_item (self);
	if (item != NULL) {
		gtk_menu_shell_select_item ((GtkMenuShell*) shell, (GtkWidget*) item);
	}
	(shell == NULL) ? NULL : (shell = (g_object_unref (shell), NULL));
	(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
}


static void __lambda1__gtk_editable_changed (GtkEntry* _sender, gpointer self) {
	_lambda1_ (_sender, self);
}


static void _lambda2_ (GtkEntry* box, SearchBoxMenuItem* self) {
	GnomenuMenu* _tmp1_;
	GtkContainer* _tmp0_;
	GnomenuMenu* shell;
	GnomenuMenuItem* item;
	g_return_if_fail (box != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	shell = (_tmp1_ = (_tmp0_ = gtk_widget_get_parent ((GtkWidget*) self), GNOMENU_IS_MENU (_tmp0_) ? ((GnomenuMenu*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_));
	item = search_box_menu_item_find_item (self);
	if (item != NULL) {
		gtk_menu_shell_activate_item ((GtkMenuShell*) shell, (GtkWidget*) item, TRUE);
	}
	(shell == NULL) ? NULL : (shell = (g_object_unref (shell), NULL));
	(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
}


static void __lambda2__gtk_entry_activate (GtkEntry* _sender, gpointer self) {
	_lambda2_ (_sender, self);
}


static void _lambda3_ (SearchBoxMenuItem* _this, GtkStateType previous_state, SearchBoxMenuItem* self) {
	g_return_if_fail (_this != NULL);
	((GtkWidget*) self)->state = (guchar) GTK_STATE_NORMAL;
}


static void __lambda3__gtk_widget_state_changed (SearchBoxMenuItem* _sender, GtkStateType previous_state, gpointer self) {
	_lambda3_ (_sender, previous_state, self);
}


static GObject * search_box_menu_item_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	SearchBoxMenuItemClass * klass;
	GObjectClass * parent_class;
	SearchBoxMenuItem * self;
	klass = SEARCH_BOX_MENU_ITEM_CLASS (g_type_class_peek (TYPE_SEARCH_BOX_MENU_ITEM));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = SEARCH_BOX_MENU_ITEM (obj);
	{
		GtkImage* _tmp20_;
		_tmp20_ = NULL;
		gtk_image_menu_item_set_image ((GtkImageMenuItem*) self, (GtkWidget*) (_tmp20_ = g_object_ref_sink ((GtkImage*) gtk_image_new_from_icon_name ("search", GTK_ICON_SIZE_MENU))));
		(_tmp20_ == NULL) ? NULL : (_tmp20_ = (g_object_unref (_tmp20_), NULL));
		g_object_set (self->priv->textbox, "editable", TRUE, NULL);
		g_object_set ((GtkWidget*) self->priv->textbox, "sensitive", TRUE, NULL);
		g_object_set ((GtkWidget*) self->priv->textbox, "can-focus", TRUE, NULL);
		gtk_entry_set_text (self->priv->textbox, "Type here...");
		g_object_set ((GtkWidget*) self->priv->textbox, "is-focus", TRUE, NULL);
		gtk_container_add ((GtkContainer*) self, (GtkWidget*) self->priv->textbox);
		gtk_widget_show_all ((GtkWidget*) self);
		gtk_container_set_focus_child ((GtkContainer*) self, (GtkWidget*) self->priv->textbox);
		((GtkWidget*) self->priv->textbox)->state = (guchar) GTK_STATE_NORMAL;
		g_signal_connect ((GtkWidget*) self->priv->textbox, "state-changed", (GCallback) __lambda0__gtk_widget_state_changed, self);
		g_signal_connect ((GtkEditable*) self->priv->textbox, "changed", (GCallback) __lambda1__gtk_editable_changed, self);
		g_signal_connect (self->priv->textbox, "activate", (GCallback) __lambda2__gtk_entry_activate, self);
		g_signal_connect ((GtkWidget*) self, "state-changed", (GCallback) __lambda3__gtk_widget_state_changed, self);
	}
	return obj;
}


static void search_box_menu_item_class_init (SearchBoxMenuItemClass * klass) {
	search_box_menu_item_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (SearchBoxMenuItemPrivate));
	GTK_WIDGET_CLASS (klass)->parent_set = search_box_menu_item_real_parent_set;
	GTK_MENU_ITEM_CLASS (klass)->activate_item = search_box_menu_item_real_activate_item;
	GTK_MENU_ITEM_CLASS (klass)->activate = search_box_menu_item_real_activate;
	G_OBJECT_CLASS (klass)->get_property = search_box_menu_item_get_property;
	G_OBJECT_CLASS (klass)->set_property = search_box_menu_item_set_property;
	G_OBJECT_CLASS (klass)->constructor = search_box_menu_item_constructor;
	G_OBJECT_CLASS (klass)->finalize = search_box_menu_item_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), SEARCH_BOX_MENU_ITEM_TEXT, g_param_spec_string ("text", "text", "text", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void search_box_menu_item_instance_init (SearchBoxMenuItem * self) {
	self->priv = SEARCH_BOX_MENU_ITEM_GET_PRIVATE (self);
	self->priv->textbox = g_object_ref_sink ((GtkEntry*) gtk_entry_new ());
}


static void search_box_menu_item_finalize (GObject* obj) {
	SearchBoxMenuItem * self;
	self = SEARCH_BOX_MENU_ITEM (obj);
	(self->priv->textbox == NULL) ? NULL : (self->priv->textbox = (g_object_unref (self->priv->textbox), NULL));
	G_OBJECT_CLASS (search_box_menu_item_parent_class)->finalize (obj);
}


GType search_box_menu_item_get_type (void) {
	static GType search_box_menu_item_type_id = 0;
	if (search_box_menu_item_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (SearchBoxMenuItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) search_box_menu_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SearchBoxMenuItem), 0, (GInstanceInitFunc) search_box_menu_item_instance_init, NULL };
		search_box_menu_item_type_id = g_type_register_static (GTK_TYPE_IMAGE_MENU_ITEM, "SearchBoxMenuItem", &g_define_type_info, 0);
	}
	return search_box_menu_item_type_id;
}


static void search_box_menu_item_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SearchBoxMenuItem * self;
	gpointer boxed;
	self = SEARCH_BOX_MENU_ITEM (object);
	switch (property_id) {
		case SEARCH_BOX_MENU_ITEM_TEXT:
		g_value_set_string (value, search_box_menu_item_get_text (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void search_box_menu_item_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SearchBoxMenuItem * self;
	self = SEARCH_BOX_MENU_ITEM (object);
	switch (property_id) {
		case SEARCH_BOX_MENU_ITEM_TEXT:
		search_box_menu_item_set_text (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


/**
 * Toolhelp routines
 *
 * */
GdkPixbuf* guess_icon (WnckWindow* window, gint width, gint height) {
	GdkPixbuf** _tmp3_;
	gint icons_size;
	gint icons_length1;
	GdkPixbuf** _tmp2_;
	GdkPixbuf* _tmp1_;
	GdkPixbuf* _tmp0_;
	GdkPixbuf** icons;
	GdkPixbuf* _tmp4_;
	g_return_val_if_fail (window != NULL, NULL);
	_tmp3_ = NULL;
	_tmp2_ = NULL;
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	icons = (_tmp3_ = (_tmp2_ = g_new0 (GdkPixbuf*, 2 + 1), _tmp2_[0] = (_tmp0_ = wnck_window_get_mini_icon (window), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), _tmp2_[1] = (_tmp1_ = wnck_window_get_icon (window), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), _tmp2_), icons_length1 = 2, icons_size = icons_length1, _tmp3_);
	_tmp4_ = NULL;
	return (_tmp4_ = guess_icon_array (icons, icons_length1, width, height), icons = (_vala_array_free (icons, icons_length1, (GDestroyNotify) g_object_unref), NULL), _tmp4_);
}


GdkPixbuf* guess_icon_array (GdkPixbuf** icons, int icons_length1, gint width, gint height) {
	GdkPixbuf* icon;
	gint min_dist;
	gint best_size;
	gboolean _tmp0_;
	gint scaled_size;
	double ratio;
	gint scaled_width;
	gint scaled_height;
	GdkPixbuf* scaled_icon;
	GdkPixbuf* _tmp6_;
	icon = NULL;
	min_dist = 99999;
	best_size = 16;
	_tmp0_ = FALSE;
	if (height == (-1)) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = width == (-1);
	}
	if (_tmp0_) {
		gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &width, &height);
	}
	scaled_size = height;
	if (width < scaled_size) {
		scaled_size = width;
	}
	{
		GdkPixbuf** i_collection;
		int i_collection_length1;
		int i_it;
		i_collection = icons;
		i_collection_length1 = icons_length1;
		for (i_it = 0; i_it < icons_length1; i_it = i_it + 1) {
			GdkPixbuf* _tmp3_;
			GdkPixbuf* i;
			_tmp3_ = NULL;
			i = (_tmp3_ = i_collection[i_it], (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_));
			{
				gint size;
				gint dist;
				size = gdk_pixbuf_get_height (i);
				if (gdk_pixbuf_get_width (i) > size) {
					size = gdk_pixbuf_get_width (i);
				}
				dist = size - scaled_size;
				if (dist < 0) {
					dist = -dist;
				}
				if (dist < min_dist) {
					GdkPixbuf* _tmp2_;
					GdkPixbuf* _tmp1_;
					min_dist = dist;
					_tmp2_ = NULL;
					_tmp1_ = NULL;
					icon = (_tmp2_ = (_tmp1_ = i, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL)), _tmp2_);
					best_size = size;
				}
				(i == NULL) ? NULL : (i = (g_object_unref (i), NULL));
			}
		}
	}
	/* This should never happen. In case it happens
	 * return something to make sure it doesn't crash*/
	if (icon == NULL) {
		GdkPixbuf* _tmp4_;
		GdkPixbuf* _tmp5_;
		_tmp4_ = NULL;
		_tmp5_ = NULL;
		return (_tmp5_ = (_tmp4_ = icons[0], (_tmp4_ == NULL) ? NULL : g_object_ref (_tmp4_)), (icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL)), _tmp5_);
	}
	ratio = ((double) scaled_size) / ((double) best_size);
	if (ratio < 0) {
		ratio = 1.0;
	}
	scaled_width = (gint) (gdk_pixbuf_get_width (icon) * ratio);
	scaled_height = (gint) (gdk_pixbuf_get_height (icon) * ratio);
	scaled_icon = gdk_pixbuf_new (GDK_COLORSPACE_RGB, gdk_pixbuf_get_has_alpha (icon), 8, scaled_width, scaled_height);
	gdk_pixbuf_scale (icon, scaled_icon, 0, 0, gdk_pixbuf_get_width (scaled_icon), gdk_pixbuf_get_height (scaled_icon), (double) 0, (double) 0, ratio, ratio, GDK_INTERP_BILINEAR);
	/* can't use scale_simple because the vala binding is wrong*/
	_tmp6_ = NULL;
	return (_tmp6_ = scaled_icon, (icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL)), _tmp6_);
}


gboolean guess_dock_is_around (void) {
	GList* windows;
	windows = wnck_screen_get_windows (wnck_screen_get_default ());
	{
		GList* window_collection;
		GList* window_it;
		window_collection = windows;
		for (window_it = window_collection; window_it != NULL; window_it = window_it->next) {
			WnckWindow* window;
			window = (WnckWindow*) window_it->data;
			{
				if (wnck_window_get_window_type (window) == WNCK_WINDOW_DOCK) {
					GQuark _tmp2_;
					const char* _tmp1_;
					_tmp1_ = NULL;
					static GQuark _tmp2__label0 = 0;
					static GQuark _tmp2__label1 = 0;
					static GQuark _tmp2__label2 = 0;
					_tmp1_ = wnck_application_get_name (wnck_window_get_application (window));
					_tmp2_ = (NULL == _tmp1_) ? 0 : g_quark_from_string (_tmp1_);
					if (((_tmp2_ == ((0 != _tmp2__label0) ? _tmp2__label0 : (_tmp2__label0 = g_quark_from_static_string ("cairo-dock")))) || (_tmp2_ == ((0 != _tmp2__label1) ? _tmp2__label1 : (_tmp2__label1 = g_quark_from_static_string ("avant-window-navigator"))))) || (_tmp2_ == ((0 != _tmp2__label2) ? _tmp2__label2 : (_tmp2__label2 = g_quark_from_static_string ("Do")))))
					do {
						return TRUE;
						break;
					} while (0);
				}
			}
		}
	}
	return FALSE;
}


WnckWindow* find_desktop (void) {
	GList* windows;
	windows = wnck_screen_get_windows (wnck_screen_get_default ());
	{
		GList* window_collection;
		GList* window_it;
		window_collection = windows;
		for (window_it = window_collection; window_it != NULL; window_it = window_it->next) {
			WnckWindow* window;
			window = (WnckWindow*) window_it->data;
			{
				if (wnck_window_get_window_type (window) == WNCK_WINDOW_DESKTOP) {
					WnckWindow* _tmp0_;
					_tmp0_ = NULL;
					return (_tmp0_ = window, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
				}
			}
		}
	}
	return NULL;
}


char* replace (const char* source, const char* find, const char* replacement) {
	char** _tmp1_;
	gint buf_size;
	gint buf_length1;
	char** _tmp0_;
	char** buf;
	GString* sb;
	const char* _tmp2_;
	char* _tmp3_;
	g_return_val_if_fail (source != NULL, NULL);
	g_return_val_if_fail (find != NULL, NULL);
	g_return_val_if_fail (replacement != NULL, NULL);
	/* replaces the string.replace method which depends on GLib.RegEx >= 2.12 */
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	buf = (_tmp1_ = _tmp0_ = g_strsplit (source, find, 0), buf_length1 = _vala_array_length (_tmp0_), buf_size = buf_length1, _tmp1_);
	sb = g_string_new ("");
	{
		gint co;
		co = 0;
		for (; co < buf_length1; co++) {
			g_string_append (sb, buf[co]);
			if (co != (buf_length1 - 1)) {
				g_string_append (sb, replacement);
			}
		}
	}
	_tmp2_ = NULL;
	_tmp3_ = NULL;
	return (_tmp3_ = (_tmp2_ = sb->str, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), buf = (_vala_array_free (buf, buf_length1, (GDestroyNotify) g_free), NULL), (sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL)), _tmp3_);
}


static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


static char* string_substring (const char* self, glong offset, glong len) {
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
}


char* remove_path (const char* txt, const char* separator) {
	glong co;
	char* ret;
	g_return_val_if_fail (txt != NULL, NULL);
	g_return_val_if_fail (separator != NULL, NULL);
	co = string_get_length (txt) - 1;
	while (TRUE) {
		gboolean _tmp0_;
		_tmp0_ = FALSE;
		if (co >= 0) {
			char* _tmp1_;
			_tmp1_ = NULL;
			_tmp0_ = _vala_strcmp0 (_tmp1_ = string_substring (txt, co, (glong) 1), separator) != 0;
			_tmp1_ = (g_free (_tmp1_), NULL);
		} else {
			_tmp0_ = FALSE;
		}
		if (!_tmp0_) {
			break;
		}
		co--;
	}
	ret = string_substring (txt, co + 1, (string_get_length (txt) - co) - 1);
	return ret;
}


char* pixbuf_encode_b64 (GdkPixbuf* pixbuf) {
	GdkPixdata pixdata = {(guint32) 0};
	guchar* _tmp1_;
	gint _tmp0_;
	g_return_val_if_fail (pixbuf != NULL, NULL);
	gdk_pixdata_from_pixbuf (&pixdata, pixbuf, TRUE);
	_tmp1_ = NULL;
	return (_tmp1_ = gdk_pixdata_serialize (&pixdata, &_tmp0_), g_base64_encode (_tmp1_, _tmp0_));
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




