
#include <glib.h>
#include <glib-object.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <gdk/gdkx.h>


#define GNOMENU_TYPE_WINDOW (gnomenu_window_get_type ())
#define GNOMENU_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GNOMENU_TYPE_WINDOW, GnomenuWindow))
#define GNOMENU_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GNOMENU_TYPE_WINDOW, GnomenuWindowClass))
#define GNOMENU_IS_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GNOMENU_TYPE_WINDOW))
#define GNOMENU_IS_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GNOMENU_TYPE_WINDOW))
#define GNOMENU_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GNOMENU_TYPE_WINDOW, GnomenuWindowClass))

typedef struct _GnomenuWindow GnomenuWindow;
typedef struct _GnomenuWindowClass GnomenuWindowClass;
typedef struct _GnomenuWindowPrivate GnomenuWindowPrivate;

/**
 * The Window widget extends Gtk.Window widget.
 *
 * The Window widget is capable of wrapping a Gdk.Window
 * or a is_foreign native window.
 *
 * It adds methods to bind any string as a property of
 * the underlining Gdk.Window.
 *
 * It emits property-notify-event signal when any properties of the underlineing Gdk.Window
 * has changed.
 *
 * As convenient wrappers,
 * it also addes the menu-context property and menu-event signal.
 *
 */
struct _GnomenuWindow {
	GObject parent_instance;
	GnomenuWindowPrivate * priv;
};

struct _GnomenuWindowClass {
	GObjectClass parent_class;
};

struct _GnomenuWindowPrivate {
	GdkWindow* _window;
	GtkWidget* key_widget;
	gboolean disposed;
	char* _menu_context;
};



GType gnomenu_window_get_type (void);
#define GNOMENU_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GNOMENU_TYPE_WINDOW, GnomenuWindowPrivate))
enum  {
	GNOMENU_WINDOW_DUMMY_PROPERTY,
	GNOMENU_WINDOW_WINDOW,
	GNOMENU_WINDOW_MENU_CONTEXT
};
void gnomenu_window_set_window (GnomenuWindow* self, GdkWindow* value);
GnomenuWindow* gnomenu_window_new (GdkWindow* window);
GnomenuWindow* gnomenu_window_construct (GType object_type, GdkWindow* window);
GnomenuWindow* gnomenu_window_new (GdkWindow* window);
GnomenuWindow* gnomenu_window_foreign_new (gulong xid);
void gnomenu_window_set_key_widget (GnomenuWindow* self, GtkWidget* widget);
static void gnomenu_window_real_dispose (GObject* base);
static GdkFilterReturn gnomenu_window_event_filter (GnomenuWindow* self, GdkXEvent* xevent, const GdkEvent* gdk_ev);
char* gnomenu_window_get_by_atom (GnomenuWindow* self, GdkAtom atom);
char* gnomenu_window_get (GnomenuWindow* self, const char* property_name);
void gnomenu_window_set_by_atom (GnomenuWindow* self, GdkAtom atom, const char* value);
void gnomenu_window_set (GnomenuWindow* self, const char* property_name, const char* value);
GdkWindow* gnomenu_window_get_window (GnomenuWindow* self);
#define GNOMENU_NET_GLOBALMENU_MENU_EVENT "_NET_GLOBALMENU_MENU_EVENT"
void gnomenu_window_emit_menu_event (GnomenuWindow* self, const char* path);
#define GNOMENU_NET_GLOBALMENU_MENU_SELECT "_NET_GLOBALMENU_MENU_SELECT"
void gnomenu_window_emit_menu_select (GnomenuWindow* self, const char* path, const char* pos);
#define GNOMENU_NET_GLOBALMENU_MENU_DESELECT "_NET_GLOBALMENU_MENU_DESELECT"
void gnomenu_window_emit_menu_deselect (GnomenuWindow* self, const char* path);
gboolean gnomenu_grab_key (GdkWindow* grab_window, guint keyval, GdkModifierType state);
gboolean gnomenu_window_grab_key (GnomenuWindow* self, guint keyval, GdkModifierType state);
gboolean gnomenu_ungrab_key (GdkWindow* grab_window, guint keyval, GdkModifierType state);
gboolean gnomenu_window_ungrab_key (GnomenuWindow* self, guint keyval, GdkModifierType state);
static GdkFilterReturn _gnomenu_window_event_filter_gdk_filter_func (GdkXEvent* xevent, const GdkEvent* event, gpointer self);
#define GNOMENU_NET_GLOBALMENU_MENU_CONTEXT "_NET_GLOBALMENU_MENU_CONTEXT"
const char* gnomenu_window_get_menu_context (GnomenuWindow* self);
void gnomenu_window_set_menu_context (GnomenuWindow* self, const char* value);
static void _lambda1_ (GnomenuWindow* t, const char* prop, GnomenuWindow* self);
static void __lambda1__gnomenu_window_property_notify_event (GnomenuWindow* _sender, const char* name, gpointer self);
static GObject * gnomenu_window_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer gnomenu_window_parent_class = NULL;
static void gnomenu_window_finalize (GObject* obj);
static void gnomenu_window_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void gnomenu_window_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);


static void g_cclosure_user_marshal_VOID__STRING_STRING (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data);

GnomenuWindow* gnomenu_window_construct (GType object_type, GdkWindow* window) {
	GParameter * __params;
	GParameter * __params_it;
	GnomenuWindow * self;
	g_return_val_if_fail (window != NULL, NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	__params_it->name = "window";
	g_value_init (&__params_it->value, GDK_TYPE_WINDOW);
	g_value_set_object (&__params_it->value, window);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


GnomenuWindow* gnomenu_window_new (GdkWindow* window) {
	return gnomenu_window_construct (GNOMENU_TYPE_WINDOW, window);
}


GnomenuWindow* gnomenu_window_foreign_new (gulong xid) {
	GdkWindow* gdk_window;
	GdkWindow* _tmp1_;
	GdkWindow* _tmp0_;
	GnomenuWindow* _tmp4_;
	gdk_window = NULL;
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	gdk_window = (_tmp1_ = (_tmp0_ = gdk_window_lookup ((GdkNativeWindow) xid), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (gdk_window == NULL) ? NULL : (gdk_window = (g_object_unref (gdk_window), NULL)), _tmp1_);
	if (gdk_window == NULL) {
		GdkWindow* _tmp3_;
		GdkWindow* _tmp2_;
		_tmp3_ = NULL;
		_tmp2_ = NULL;
		gdk_window = (_tmp3_ = (_tmp2_ = gdk_window_foreign_new ((GdkNativeWindow) xid), (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_)), (gdk_window == NULL) ? NULL : (gdk_window = (g_object_unref (gdk_window), NULL)), _tmp3_);
	}
	_tmp4_ = NULL;
	return (_tmp4_ = gnomenu_window_new (gdk_window), (gdk_window == NULL) ? NULL : (gdk_window = (g_object_unref (gdk_window), NULL)), _tmp4_);
}


/*must be a toplevel */
void gnomenu_window_set_key_widget (GnomenuWindow* self, GtkWidget* widget) {
	GtkWidget* _tmp1_;
	GtkWidget* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->key_widget = (_tmp1_ = (_tmp0_ = widget, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->key_widget == NULL) ? NULL : (self->priv->key_widget = (g_object_unref (self->priv->key_widget), NULL)), _tmp1_);
}


static void gnomenu_window_real_dispose (GObject* base) {
	GnomenuWindow * self;
	self = (GnomenuWindow*) base;
	if (!self->priv->disposed) {
		self->priv->disposed = TRUE;
		gnomenu_window_set_window (self, NULL);
	}
}


static GdkFilterReturn gnomenu_window_event_filter (GnomenuWindow* self, GdkXEvent* xevent, const GdkEvent* gdk_ev) {
	XAnyEvent* event;
	GdkEvent ev;
	g_return_val_if_fail (self != NULL, 0);
	g_return_val_if_fail (xevent != NULL, 0);
	if (self->priv->disposed) {
		g_critical ("window.vala:78: event_filter invoked on a disposed window");
		return GDK_FILTER_CONTINUE;
	}
	event = (XAnyEvent*) xevent;
	ev = (*gdk_ev);
	/* copy the gdk_event*/
	switch (event->type) {
		case PropertyNotify:
		{
			char* _tmp1_;
			ev.type = GDK_PROPERTY_NOTIFY;
			ev.property.atom = gdk_x11_xatom_to_atom (((XPropertyEvent*) event)->atom);
			ev.property.time = ((XPropertyEvent*) event)->time;
			ev.property.state = (guint) ((XPropertyEvent*) event)->state;
			_tmp1_ = NULL;
			g_signal_emit_by_name (self, "property-notify-event", _tmp1_ = gdk_atom_name (ev.property.atom));
			_tmp1_ = (g_free (_tmp1_), NULL);
			break;
		}
		case KeyPress:
		{
			gboolean _tmp2_;
			_tmp2_ = FALSE;
			if (self->priv->key_widget != NULL) {
				_tmp2_ = self->priv->key_widget->window != NULL;
			} else {
				_tmp2_ = FALSE;
			}
			if (_tmp2_) {
				GdkWindow* _tmp3_;
				GdkWindow* gwin;
				Window xwin;
				Display* xd;
				/* Send a Fake key press event to the key widget
				 * if it is realized. */
				_tmp3_ = NULL;
				gwin = (_tmp3_ = self->priv->key_widget->window, (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_));
				xwin = GDK_WINDOW_XWINDOW (gwin);
				xd = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
				event->window = xwin;
				XSendEvent (xd, xwin, FALSE, (glong) 0, event);
				(gwin == NULL) ? NULL : (gwin = (g_object_unref (gwin), NULL));
			}
			break;
		}
	}
	return GDK_FILTER_CONTINUE;
}


char* gnomenu_window_get (GnomenuWindow* self, const char* property_name) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (property_name != NULL, NULL);
	return gnomenu_window_get_by_atom (self, gdk_atom_intern (property_name, FALSE));
}


void gnomenu_window_set (GnomenuWindow* self, const char* property_name, const char* value) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (property_name != NULL);
	gnomenu_window_set_by_atom (self, gdk_atom_intern (property_name, FALSE), value);
}


char* gnomenu_window_get_by_atom (GnomenuWindow* self, GdkAtom atom) {
	char* context;
	GdkAtom actual_type;
	GdkAtom type;
	gint actual_format;
	gint actual_length;
	g_return_val_if_fail (self != NULL, NULL);
	context = NULL;
	type = gdk_atom_intern ("STRING", FALSE);
	actual_format = 0;
	actual_length = 0;
	gdk_property_get (gnomenu_window_get_window (self), atom, type, (gulong) 0, (gulong) G_MAXLONG, FALSE, &actual_type, &actual_format, &actual_length, &context);
	return context;
}


void gnomenu_window_set_by_atom (GnomenuWindow* self, GdkAtom atom, const char* value) {
	g_return_if_fail (self != NULL);
	if (value != NULL) {
		GdkAtom type;
		type = gdk_atom_intern ("STRING", FALSE);
		gdk_property_change (gnomenu_window_get_window (self), atom, type, 8, GDK_PROP_MODE_REPLACE, value, ((gint) strlen (value)) + 1);
	} else {
		gdk_property_delete (gnomenu_window_get_window (self), atom);
	}
}


/**
 * emitted when a menu item is activated
 */
void gnomenu_window_emit_menu_event (GnomenuWindow* self, const char* path) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (path != NULL);
	gnomenu_window_set (self, GNOMENU_NET_GLOBALMENU_MENU_EVENT, path);
}


/**
 * emitted when a menu item is selected 
 */
void gnomenu_window_emit_menu_select (GnomenuWindow* self, const char* path, const char* pos) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (path != NULL);
	if (pos != NULL) {
		char* _tmp1_;
		char* _tmp0_;
		_tmp1_ = NULL;
		_tmp0_ = NULL;
		gnomenu_window_set (self, GNOMENU_NET_GLOBALMENU_MENU_SELECT, _tmp1_ = g_strconcat (_tmp0_ = g_strconcat (path, "@", NULL), pos, NULL));
		_tmp1_ = (g_free (_tmp1_), NULL);
		_tmp0_ = (g_free (_tmp0_), NULL);
	} else {
		gnomenu_window_set (self, GNOMENU_NET_GLOBALMENU_MENU_SELECT, path);
	}
}


/**
 * emitted when a menu item is deselected 
 */
void gnomenu_window_emit_menu_deselect (GnomenuWindow* self, const char* path) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (path != NULL);
	gnomenu_window_set (self, GNOMENU_NET_GLOBALMENU_MENU_DESELECT, path);
}


/**
 * globally grab a key to this window.
 *
 * return false if failed.
 */
gboolean gnomenu_window_grab_key (GnomenuWindow* self, guint keyval, GdkModifierType state) {
	g_return_val_if_fail (self != NULL, FALSE);
	return gnomenu_grab_key (gnomenu_window_get_window (self), keyval, state);
}


/**
 * globally ungrab a grabbed a key to this window
 * return false if failed.
 */
gboolean gnomenu_window_ungrab_key (GnomenuWindow* self, guint keyval, GdkModifierType state) {
	g_return_val_if_fail (self != NULL, FALSE);
	return gnomenu_ungrab_key (gnomenu_window_get_window (self), keyval, state);
}


GdkWindow* gnomenu_window_get_window (GnomenuWindow* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_window;
}


static GdkFilterReturn _gnomenu_window_event_filter_gdk_filter_func (GdkXEvent* xevent, const GdkEvent* event, gpointer self) {
	return gnomenu_window_event_filter (self, xevent, event);
}


void gnomenu_window_set_window (GnomenuWindow* self, GdkWindow* value) {
	GdkWindow* _tmp2_;
	GdkWindow* _tmp1_;
	g_return_if_fail (self != NULL);
	if (self->priv->_window != NULL) {
		gdk_window_remove_filter (self->priv->_window, _gnomenu_window_event_filter_gdk_filter_func, self);
	}
	_tmp2_ = NULL;
	_tmp1_ = NULL;
	self->priv->_window = (_tmp2_ = (_tmp1_ = value, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL)), _tmp2_);
	if (self->priv->_window != NULL) {
		gdk_window_add_filter (self->priv->_window, _gnomenu_window_event_filter_gdk_filter_func, self);
	}
	g_object_notify ((GObject *) self, "window");
}


const char* gnomenu_window_get_menu_context (GnomenuWindow* self) {
	char* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = NULL;
	self->priv->_menu_context = (_tmp0_ = gnomenu_window_get (self, GNOMENU_NET_GLOBALMENU_MENU_CONTEXT), self->priv->_menu_context = (g_free (self->priv->_menu_context), NULL), _tmp0_);
	return self->priv->_menu_context;
}


void gnomenu_window_set_menu_context (GnomenuWindow* self, const char* value) {
	g_return_if_fail (self != NULL);
	gnomenu_window_set (self, GNOMENU_NET_GLOBALMENU_MENU_CONTEXT, value);
	g_object_notify ((GObject *) self, "menu-context");
}


static void _lambda1_ (GnomenuWindow* t, const char* prop, GnomenuWindow* self) {
	g_return_if_fail (t != NULL);
	g_return_if_fail (prop != NULL);
	if (_vala_strcmp0 (prop, GNOMENU_NET_GLOBALMENU_MENU_CONTEXT) == 0) {
		g_signal_emit_by_name (self, "menu-context-changed");
	}
	if (_vala_strcmp0 (prop, GNOMENU_NET_GLOBALMENU_MENU_EVENT) == 0) {
		char* _tmp0_;
		_tmp0_ = NULL;
		g_signal_emit_by_name (self, "menu-event", _tmp0_ = gnomenu_window_get (self, GNOMENU_NET_GLOBALMENU_MENU_EVENT));
		_tmp0_ = (g_free (_tmp0_), NULL);
	}
	if (_vala_strcmp0 (prop, GNOMENU_NET_GLOBALMENU_MENU_SELECT) == 0) {
		char* value;
		char** _tmp2_;
		gint arr_size;
		gint arr_length1;
		char** _tmp1_;
		char** arr;
		gboolean _tmp3_;
		value = gnomenu_window_get (self, GNOMENU_NET_GLOBALMENU_MENU_SELECT);
		_tmp2_ = NULL;
		_tmp1_ = NULL;
		arr = (_tmp2_ = _tmp1_ = g_strsplit (value, "@", 0), arr_length1 = _vala_array_length (_tmp1_), arr_size = arr_length1, _tmp2_);
		_tmp3_ = FALSE;
		if (arr != NULL) {
			_tmp3_ = arr_length1 >= 1;
		} else {
			_tmp3_ = FALSE;
		}
		if (_tmp3_) {
			g_signal_emit_by_name (self, "menu-select", arr[0], arr[1]);
		}
		value = (g_free (value), NULL);
		arr = (_vala_array_free (arr, arr_length1, (GDestroyNotify) g_free), NULL);
	}
	if (_vala_strcmp0 (prop, GNOMENU_NET_GLOBALMENU_MENU_DESELECT) == 0) {
		char* value;
		value = gnomenu_window_get (self, GNOMENU_NET_GLOBALMENU_MENU_DESELECT);
		g_signal_emit_by_name (self, "menu-deselect", value);
		value = (g_free (value), NULL);
	}
}


static void __lambda1__gnomenu_window_property_notify_event (GnomenuWindow* _sender, const char* name, gpointer self) {
	_lambda1_ (_sender, name, self);
}


static GObject * gnomenu_window_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GnomenuWindowClass * klass;
	GObjectClass * parent_class;
	GnomenuWindow * self;
	klass = GNOMENU_WINDOW_CLASS (g_type_class_peek (GNOMENU_TYPE_WINDOW));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = GNOMENU_WINDOW (obj);
	{
		g_signal_connect (self, "property-notify-event", (GCallback) __lambda1__gnomenu_window_property_notify_event, self);
	}
	return obj;
}


static void gnomenu_window_class_init (GnomenuWindowClass * klass) {
	gnomenu_window_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GnomenuWindowPrivate));
	G_OBJECT_CLASS (klass)->dispose = gnomenu_window_real_dispose;
	G_OBJECT_CLASS (klass)->get_property = gnomenu_window_get_property;
	G_OBJECT_CLASS (klass)->set_property = gnomenu_window_set_property;
	G_OBJECT_CLASS (klass)->constructor = gnomenu_window_constructor;
	G_OBJECT_CLASS (klass)->finalize = gnomenu_window_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GNOMENU_WINDOW_WINDOW, g_param_spec_object ("window", "window", "window", GDK_TYPE_WINDOW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GNOMENU_WINDOW_MENU_CONTEXT, g_param_spec_string ("menu-context", "menu-context", "menu-context", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_signal_new ("property_notify_event", GNOMENU_TYPE_WINDOW, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
	g_signal_new ("menu_context_changed", GNOMENU_TYPE_WINDOW, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("menu_event", GNOMENU_TYPE_WINDOW, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
	g_signal_new ("menu_select", GNOMENU_TYPE_WINDOW, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__STRING_STRING, G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_STRING);
	g_signal_new ("menu_deselect", GNOMENU_TYPE_WINDOW, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
}


static void gnomenu_window_instance_init (GnomenuWindow * self) {
	self->priv = GNOMENU_WINDOW_GET_PRIVATE (self);
	self->priv->disposed = FALSE;
}


static void gnomenu_window_finalize (GObject* obj) {
	GnomenuWindow * self;
	self = GNOMENU_WINDOW (obj);
	(self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL));
	(self->priv->key_widget == NULL) ? NULL : (self->priv->key_widget = (g_object_unref (self->priv->key_widget), NULL));
	self->priv->_menu_context = (g_free (self->priv->_menu_context), NULL);
	G_OBJECT_CLASS (gnomenu_window_parent_class)->finalize (obj);
}


GType gnomenu_window_get_type (void) {
	static GType gnomenu_window_type_id = 0;
	if (gnomenu_window_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GnomenuWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gnomenu_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GnomenuWindow), 0, (GInstanceInitFunc) gnomenu_window_instance_init, NULL };
		gnomenu_window_type_id = g_type_register_static (G_TYPE_OBJECT, "GnomenuWindow", &g_define_type_info, 0);
	}
	return gnomenu_window_type_id;
}


static void gnomenu_window_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GnomenuWindow * self;
	gpointer boxed;
	self = GNOMENU_WINDOW (object);
	switch (property_id) {
		case GNOMENU_WINDOW_WINDOW:
		g_value_set_object (value, gnomenu_window_get_window (self));
		break;
		case GNOMENU_WINDOW_MENU_CONTEXT:
		g_value_set_string (value, gnomenu_window_get_menu_context (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gnomenu_window_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GnomenuWindow * self;
	self = GNOMENU_WINDOW (object);
	switch (property_id) {
		case GNOMENU_WINDOW_WINDOW:
		gnomenu_window_set_window (self, g_value_get_object (value));
		break;
		case GNOMENU_WINDOW_MENU_CONTEXT:
		gnomenu_window_set_menu_context (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}



static void g_cclosure_user_marshal_VOID__STRING_STRING (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data) {
	typedef void (*GMarshalFunc_VOID__STRING_STRING) (gpointer data1, const char* arg_1, const char* arg_2, gpointer data2);
	register GMarshalFunc_VOID__STRING_STRING callback;
	register GCClosure * cc;
	register gpointer data1, data2;
	cc = (GCClosure *) closure;
	g_return_if_fail (n_param_values == 3);
	if (G_CCLOSURE_SWAP_DATA (closure)) {
		data1 = closure->data;
		data2 = param_values->data[0].v_pointer;
	} else {
		data1 = param_values->data[0].v_pointer;
		data2 = closure->data;
	}
	callback = (GMarshalFunc_VOID__STRING_STRING) (marshal_data ? marshal_data : cc->callback);
	callback (data1, g_value_get_string (param_values + 1), g_value_get_string (param_values + 2), data2);
}



